package com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.service;

import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.service.CrudService;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.entity.ReservePreconstruction;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.mapper.ReservePreconstructMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * 储备项目预土建制化信息 service
 * @author: 王强
 * @create: 2022-07-26 09:40
 **/
@Service
@Transactional(readOnly = true)
public class ReservePreconstructService extends CrudService<ReservePreconstructMapper, ReservePreconstruction> {

    @Autowired
    private ReservePreconstructMapper dao;

    /**
     * 查询已有的参数数据 并分类返回已存在数据和未存在数据
     * @param reserveManagement
     * @return
     */
    public Map<String, List<ReservePreconstruction>> getExistProjectList(List<ReservePreconstruction> reserveManagement){
        Map<String,List<ReservePreconstruction>> map = new HashMap();
        List<ReservePreconstruction> existReserveManagementList = new ArrayList<>();
        Iterator<ReservePreconstruction> iterator = reserveManagement.iterator();
        while (iterator.hasNext()) {
            ReservePreconstruction rm = iterator.next();
            //对数据进行处理
            rm.preInsert();
            ReservePreconstruction reserveManag = dao.queryByProjectReserveId(rm);
            //数据库中已含有的数据在list中删除并创建一个list加入 区分为已存在数据
            if (null != reserveManag) {
                rm.setId(reserveManag.getId());
                existReserveManagementList.add(rm);
                iterator.remove();//使用迭代器删除已存在的数据
            }
        }
        map.put("without",reserveManagement);
        map.put("exist",existReserveManagementList);
        return map;
    }

    /**
     * 批量添加数据
     * @param reserveManagementList
     */
    @Transactional(readOnly = false)
    public void insertList(List<ReservePreconstruction> reserveManagementList){

        dao.insertList(reserveManagementList);
    }

    /**
     * 专职修改方法
     * @param reserveManagement
     */
    @Transactional(readOnly = false)
    public void update(ReservePreconstruction reserveManagement) {

        super.save(reserveManagement);
    }

    public Page<ReservePreconstruction> findPage(Page<ReservePreconstruction> page, ReservePreconstruction reserveManagement) {
        Page<ReservePreconstruction> pageList = super.findPage(page, reserveManagement);
        return pageList;
    }

    public List<ReservePreconstruction> exportFindList( ReservePreconstruction reserveManagement) {
        List<ReservePreconstruction> pageList = dao.exportFindList(reserveManagement);
        return pageList;
    }

    /**
     * 新增方法
     * @param entity
     */
    @Transactional(readOnly = false)
    public void save(ReservePreconstruction entity) {
        //查询数据表中是否含有同项目储备编号的数据 若有则进行修改 没有则新增
        //根据项目储备编号 查询数据
        if (entity.getIsNewRecord()){
            entity.preInsert();
            mapper.insert(entity);
        }else{
            entity.preUpdate();
            mapper.update(entity);
        }
    }
}
