package com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.service;

import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.service.CrudService;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.entity.ReserveProcessProject;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.mapper.ReserveProcessProjectMapper;
import com.jeeplus.modules.sg.managementcenter.reserveManagement.entity.ProgressSchedule;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 储备项目基本信息 service
 * @author: 王强
 * @create: 2022-07-20 16:37
 **/
@Service
@Transactional(readOnly = true)
public class ReserveProcessProjectService extends CrudService<ReserveProcessProjectMapper, ReserveProcessProject> {

    @Autowired
    private ReserveProcessProjectMapper dao;

    /**
     * 根据项目储备id查询项目信息
     * @param reserveManagement
     * @return
     */
    public ReserveProcessProject queryByProjectReserveId(ReserveProcessProject reserveManagement){
        return dao.queryByProjectReserveId(reserveManagement);
    }

    /**
     * 根据项目储备id查询项目信息
     * @param projectReserveId
     * @return
     */
    public ReserveProcessProject queryByProjectId(String projectReserveId){
        return dao.queryByProjectId(projectReserveId);
    }

    /**
     * 查看数据库唯一性判断字段值
     * @return
     */
    public List<String> repeatList(){
        return dao.repeatList();
    }

    public Page<ReserveProcessProject> findPage(Page<ReserveProcessProject> page, ReserveProcessProject reserveManagement) {
        Page<ReserveProcessProject> pageList = super.findPage(page, reserveManagement);
        return pageList;
    }

    /**
     * 查询已有的参数数据 并分类返回已存在数据和未存在数据
     * @param reserveManagement
     * @return
     */
    public Map<String,List<ReserveProcessProject>> getExistProjectList(List<ReserveProcessProject> reserveManagement){
        Map<String,List<ReserveProcessProject>> map = new HashMap();
        List<ReserveProcessProject> existReserveManagementList = new ArrayList<>();
        Iterator<ReserveProcessProject> iterator = reserveManagement.iterator();
        while (iterator.hasNext()) {
            ReserveProcessProject rm = iterator.next();
            //对数据进行处理
            rm.preInsert();
            ReserveProcessProject reserveManag = dao.queryByProjectReserveId(rm);

            //数据库中已含有的数据在list中删除并创建一个list加入 区分为已存在数据
            if (null != reserveManag) {
                rm.setId(reserveManag.getId());
                existReserveManagementList.add(rm);
                iterator.remove();//使用迭代器删除已存在的数据
            }
        }
        map.put("without",reserveManagement);
        map.put("exist",existReserveManagementList);
        return map;
    }

    /**
     * 批量添加数据
     * @param reserveManagementList
     */
    @Transactional(readOnly = false)
    public void insertList(List<ReserveProcessProject> reserveManagementList){
        SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String format = dft.format(new Date());
        ProgressSchedule schedule = new ProgressSchedule();
        for (ReserveProcessProject a :
                reserveManagementList) {
            a.preInsert();
            schedule.preInsert();
            schedule.setProjectBasic(format);
            schedule.setProjectReserveId(a.getProjectReserveId());
        }
        dao.insertList(reserveManagementList);
    }

    /**
     * 新增方法
     * @param entity
     */
    @Transactional(readOnly = false)
    public void save(ReserveProcessProject entity) {
        //查询数据表中是否含有同项目储备编号的数据 若有则进行修改 没有则新增
        //根据项目储备编号 查询数据
        if (entity.getIsNewRecord()){
            entity.preInsert();
            mapper.insert(entity);
        }else{
            entity.preUpdate();
            mapper.update(entity);
        }
    }

    /**
     * 专职修改方法
     * @param reserveManagement
     */
    @Transactional(readOnly = false)
    public void update(ReserveProcessProject reserveManagement) {

        super.save(reserveManagement);
    }

    public List<ReserveProcessProject> exportFindList( ReserveProcessProject reserveManagement) {
        List<ReserveProcessProject> pageList = dao.exportFindList(reserveManagement);
        return pageList;
    }

    /**
     * 删除方法
     * @param reserveManagement
     */
    @Transactional(readOnly = false)
    public void delete(ReserveProcessProject reserveManagement) {
        super.delete(reserveManagement);
    }

    public Page<ReserveProcessProject> findModulePage (Page<ReserveProcessProject> page, ReserveProcessProject entity) {
        dataRuleFilter(entity);
        entity.setPage(page);
        page.setList(mapper.findModuleList(entity));
        return page;
    }

    /**
     * 经理修改方法
     * @param reserveManagement
     */
    @Transactional(readOnly = false)
    public void updatejl(ReserveProcessProject reserveManagement) {
        reserveManagement.preUpdate();
        dao.updatejl(reserveManagement);
    }

    @Transactional(readOnly = false)
    public void jlSave(ReserveProcessProject scheme) {
        //查询数据表中是否含有同项目储备编号的数据 若有则进行修改
        //根据项目储备编号 查询数据
        Integer byProjectId = mapper.findByProjectId(scheme.getProjectReserveId());
        SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String format = dft.format(new Date());
        ProgressSchedule schedule = new ProgressSchedule();
        //如果存在则修改信息
        if (null != byProjectId && 1 == byProjectId) {
            scheme.preUpdate();

            mapper.updateScheme(scheme);

        }
    }

    /**
     * 通过key值查找对应的流程id
     */
    public String findDefId(String key) {
        return dao.findDefId(key);
    }

    /**
     * 通过流程定实例id查询储备人员信息
     */
    public ReserveProcessProject findCgUser(String procInsId) {
        return dao.findCgUser(procInsId);
    }
}
