package com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.service;

import com.google.common.collect.Maps;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.service.CrudService;
import com.jeeplus.modules.act.entity.Act;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.entity.*;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.mapper.ReserveSummaryProcessMapper;
import com.jeeplus.modules.sg.managementcenter.activiti.utils.MyActiviUtils;
import com.jeeplus.modules.sg.managementcenter.materialinfo.entity.MaterialInfo;
import com.jeeplus.modules.sg.managementcenter.materialinfo.util.MaterialInfoUtil;
import com.jeeplus.modules.sys.utils.UserUtils;
import org.activiti.engine.TaskService;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.Row;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author: 王强
 * @create: 2022-08-03 10:15
 **/
@Service
@Transactional(readOnly = true)
public class ReserveSummaryProcessService extends CrudService<ReserveSummaryProcessMapper, ReserveSummaryProcess> {

    @Autowired
    private TaskService taskService;

    @Autowired
    private SummaryStatisticService statisticService;

    @Autowired
    private BudgetBookService budgetBookService;

    @Autowired
    private FeedBackBudgetService budgetService;



    @Transactional(readOnly = false)
    public void insert(ReserveSummaryProcess entity) {
        mapper.insert(entity);
    }

    @Transactional(readOnly = false)
    public void deleteAct(String projectReserveId) {
        mapper.deleteAct(projectReserveId);
    }



    @Override
    public ReserveSummaryProcess get(String id) {
        ReserveSummaryProcess entity = new ReserveSummaryProcess();
        entity.setId(id);
        String projectId = mapper.findProjectId(entity);
        ReserveSummaryProcess reserveProcess = mapper.get(projectId);
        return reserveProcess;
    }

    /**
     *在年度物资上报统计表中保存项目信息，在年度上报明细表中保存物资信息
     */
    @Transactional(readOnly = false)
    public void saveData(MultipartFile file) throws IOException, InvalidFormatException {
        ImportExcel module = new ImportExcel(file,1,"模块表");
        Row row = module.getRow(0);
        Object obj = module.getCellValue(row, 1);
        if(obj==null){
            throw new RuntimeException("找不到项目名称！");
        }
        String projectName = ((String) obj).trim();
        //删除对应项目名称的甲供材信息
        mapper.deleteMaterial(projectName);
        ImportExcel pick = new ImportExcel(file,7,"领料清单");
        List<MaterialInfo> materialInfos = MaterialInfoUtil.getCbData(pick,projectName);   //获取领料清单数据
        List<MaterialInfo> list = MaterialInfoUtil.combineData(materialInfos);      //合并数据
        //通过甲供材信息比对物料规范获得项目建设规模
        saveList(list);
    }

    @Transactional(readOnly = false)
    public void saveList(List<MaterialInfo> list) {
        for(MaterialInfo maintainData : list){
            maintainData.preInsert();
        }
        mapper.insertList(list);
    }

    private String returnKva(String materialDescription) {
        if (null != materialDescription && !"".equals(materialDescription)) {
            boolean kVA = materialDescription.contains("kVA");
            if (kVA) {
                int kVA1 = materialDescription.lastIndexOf("kVA");
                String substring = materialDescription.substring(kVA1 - 3, kVA1+3);
                return substring;
            }
        }
        return "";
    }

    /**
     * 通过储备项目编号查询项目是否有甲供材清册
     */
    public Integer findMaterialData(String procInsId) {
        return mapper.findMaterialData(procInsId);
    }

    /**
     * 修改项目设计单位信息
     */
    @Transactional(readOnly = false)
    public void updateDesign(ReserveSummaryProcess reserveProcess) {
        mapper.updateDesign(reserveProcess);
    }

    /**
     * 设计单位判定是否关闭
     */
    @Transactional(readOnly = false)
    public void actDesignUnit(Act act, ReserveSummaryProcess reserveProcess) {

        // 对不同环节的业务逻辑进行操作
        String taskDefKey = act.getTaskDefKey();
        //提交流程任务
        Map<String, Object> vars = Maps.newHashMap();

        ReserveProcessProject user = MyActiviUtils.findCgUser(act.getProcInsId());

        //候选人
        List list = new ArrayList<>();
        list.add(UserUtils.getByUserName(user.getOwnerProjectManager()).getLoginName());
        list.add(UserUtils.getByUserName(user.getOwnerReserveFullTime()).getLoginName());

        vars.put("person",list);

        //指定代理人
        vars.put("manager",UserUtils.getByUserName(user.getOwnerProjectManager()).getLoginName());


        // 提交流程任务
        complete(act.getTaskId(), act.getProcInsId(), act.getComment(), vars);
    }

    /**
     * 提交任务, 并保存意见
     *
     * @param taskId    任务ID
     * @param procInsId 流程实例ID，如果为空，则不保存任务提交意见
     * @param comment   任务提交意见的内容
     * @param vars      任务变量
     */
    @Transactional(readOnly = false)
    public void complete(String taskId, String procInsId, String comment, Map<String, Object> vars) {
        complete(taskId, procInsId, comment, "", vars);
    }

    @Transactional(readOnly = false)
    public void complete(String taskId, String procInsId, String comment, String title, Map<String, Object> vars) {
        // 添加意见
        if (StringUtils.isNotBlank(procInsId) && StringUtils.isNotBlank(comment)) {
            taskService.addComment(taskId, procInsId, comment);
        }

        // 设置流程变量
        if (vars == null) {
            vars = Maps.newHashMap();
        }

        // 设置流程标题
        if (StringUtils.isNotBlank(title)) {
            vars.put("title", title);
        }

        // 提交任务
        taskService.complete(taskId, vars);
    }

    public Page<ReserveSummaryProcess> findAllUploadPage(Page<ReserveSummaryProcess> page, ReserveSummaryProcess entity) {
        dataRuleFilter(entity);
        entity.setPage(page);
        page.setList(mapper.findAllUpload(entity));
        return page;
    }

    public Page<MaterialInfo> findMaterialPage(Page<MaterialInfo> page, MaterialInfo entity) {
        dataRuleFilter(entity);
        entity.setPage(page);
        page.setList(mapper.findMaterialList(entity));
        return page;
    }

    /**
     * 通过项目名称导出符合条件的数据
     */
    public List<MaterialInfo> findMaterialActiviti(String projectName) {
        return mapper.findMaterialActiviti(projectName);
    }

    /**
     * 项目经理判定是否通过
     */
    @Transactional(readOnly = false)
    public void actDrawingReview(Act act) {
        Map<String, Object> vars = Maps.newHashMap();
        String flag = act.getFlag();
        ReserveProcessProject user = MyActiviUtils.findCgUser(act.getProcInsId());

        //签收
        /*taskService.claim(act.getTaskId(),UserUtils.getByUserName(user.getOwnerProjectManager()).getId());
        logger.info(taskService.createTaskQuery().singleResult().getAssignee());
        act.preUpdate();*/

        if ("yes".equals(flag)) {
            //通过流程到-设计人员上传预算书、项目建议书/可研报告、设计说明书
            vars.put("manager", UserUtils.getByUserName(user.getDesigner()).getLoginName());
            vars.put("pass",true);
            vars.put("no",false);
            vars.put("rural",false);
            vars.put("tg",false);
        } else if ("disagree".equals(flag)){
            //不通过-回到上一个流程
            vars.put("manager", UserUtils.getByUserName(user.getDesigner()).getLoginName());
            vars.put("pass",false);
            vars.put("tg",true);
            vars.put("no",false);
            vars.put("rural",false);

        } else if ("cancel".equals(flag)){
            //取消-项目取消，修改第六张表的状态
            //先把表的状态改为取消
            ReserveSummary reserveSummary = new ReserveSummary();
            reserveSummary.setProjectReserveId(user.getProjectReserveId());
            ReserveSummary summary = statisticService.queryByProjectReserveId(reserveSummary);
            summary.setIncludeReserve("取消");
            statisticService.update(summary);
            vars.put("tg",false);
            vars.put("pass",false);
            vars.put("no",true);
            vars.put("rural",false);

        } else if ("pending".equals(flag)){
            //待定 修改第六张表的状态
            ReserveSummary reserveSummary = new ReserveSummary();
            reserveSummary.setProjectReserveId(user.getProjectReserveId());
            ReserveSummary summary = statisticService.queryByProjectReserveId(reserveSummary);
            summary.setIncludeReserve("待定");
            statisticService.update(summary);
            vars.put("tg",false);
            vars.put("pass",false);
            vars.put("no",false);
            vars.put("rural",true);

        }

        act.preUpdate();
        // 对不同环节的业务逻辑进行操作
        String taskDefKey = act.getTaskDefKey();
        //查询出未完成的任务
        /*List<Task> list = taskService.createTaskQuery().processInstanceId(act.getProcInsId()).list();
        //将联合审图中未完成的任务设置为完成状态
        taskService.complete(list.get(0).getId());*/
        //完成后不显示给该用户
//        taskService.deleteTask(list.get(0).getId(),true);


        // 提交流程任务
        complete(act.getTaskId(), act.getProcInsId(), act.getComment(), vars);
    }

    /**
     * 技经人员判定是否关闭
     */
    @Transactional(readOnly = false)
    public void actBudgetUnit(Act act,ReserveSummaryProcess reserveProcess) {
        Map<String, Object> vars = Maps.newHashMap();
        String flag = act.getFlag();
        ReserveProcessProject user = MyActiviUtils.findCgUser(act.getProcInsId());

        String projectReserveId = "";
        String budgetRevision = "";
        //修改储备项目概预算统计表的-是否完成概算修改 和 概算修改完成时间字段
        if ("yes".equals(flag)) {
            vars.put("pass",true);
            //根据项目储备编号修改 是否完成概算修改状态
            projectReserveId = reserveProcess.getProjectReserveId();
            budgetRevision = "无需修改";

            //将预算书审前审后数据放入数据库
            BudgetBook budgetBook = new BudgetBook();
            budgetBook.setProjectReserveId(reserveProcess.getProjectReserveId());
            budgetBook.setPretrialBudgetTaxInclude(reserveProcess.getPretrialBudgetTaxInclude());
            budgetBook.setPretrialBudgetTaxNoInclude(reserveProcess.getPretrialBudgetTaxNoInclude());
            budgetBook.setPretrialSupervisionFeeTaxNoInclude(reserveProcess.getPretrialSupervisionFeeTaxNoInclude());
            budgetBook.setPretrialDesignFeeTaxNoInclude(reserveProcess.getPretrialDesignFeeTaxNoInclude());
            budgetBook.setAfterTrialBudgetTaxInclude(reserveProcess.getAfterTrialBudgetTaxInclude());
            budgetBook.setAfterTrialBudgetTaxNoInclude(reserveProcess.getAfterTrialBudgetTaxNoInclude());
            budgetBook.setAfterTrialSupervisionFeeTaxNoInclude(reserveProcess.getAfterTrialSupervisionFeeTaxNoInclude());
            budgetBook.setAfterTrialDesignFeeTaxNoInclude(reserveProcess.getAfterTrialDesignFeeTaxNoInclude());
            //插入前先清空该项目相关的预算书数据
            budgetBookService.deleteByProjectReserveId(budgetBook.getProjectReserveId());
            //插入
            budgetBookService.save(budgetBook);
        } else if ("no".equals(flag)){
            //不通过-回到上一个流程
            vars.put("manager", UserUtils.getByUserName(user.getDesigner()).getLoginName());
            vars.put("pass",false);
            //根据项目储备编号修改 是否完成概算修改状态
            projectReserveId = reserveProcess.getProjectReserveId();
            budgetRevision = "修改中";

            //将预算问题反馈记录保存到数据库
            FeedBackBudget budget = new FeedBackBudget();
            budget.setProjectReserveId(reserveProcess.getProjectReserveId());
            budget.setPreferedCase(reserveProcess.getPreferedCase());
            budget.setTransferSituation(reserveProcess.getTransferSituation());
            budget.setBudgetTemplateCheck(reserveProcess.getBudgetTemplateCheck());
            budget.setQuotaApplicationCheck(reserveProcess.getQuotaApplicationCheck());
            budget.setWorkLoadCheck(reserveProcess.getWorkLoadCheck());
            budget.setCheckTheFee(reserveProcess.getCheckTheFee());
            budget.setSupplyCheck(reserveProcess.getSupplyCheck());
            budget.setMaterialCheck(reserveProcess.getMaterialCheck());
            budget.setLiveWorkFeeCheck(reserveProcess.getLiveWorkFeeCheck());
            budget.setDemolitionEngineeringProblems(reserveProcess.getDemolitionEngineeringProblems());
            budget.setOtherProblem(reserveProcess.getOtherProblem());

            budgetService.deleteByProjectReserveId(budget.getProjectReserveId());
            budgetService.save(budget);
        }

        act.preUpdate();
        // 对不同环节的业务逻辑进行操作
        String taskDefKey = act.getTaskDefKey();

        // 提交流程任务
        complete(act.getTaskId(), act.getProcInsId(), act.getComment(), vars);
        //设置完成时间
        Date date = new Date();
        DateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String time = format.format(date);

    }
}
