package com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.web;

import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.entity.ReservePreconstruction;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.service.ReservePreconstructService;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.utils.ReserveProcessProjectExportUtil;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.utils.ReserveProcessProjectProUtil;
import com.jeeplus.modules.sg.managementcenter.materialproject.util.ImportUtil;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**
 * 储备项目预土建制化信息 controller
 * @author: 王强
 * @create: 2022-07-26 09:22
 **/
@Controller
@RequestMapping(value = "${adminPath}/cg/reservemanagementcenter/reservePreconstruct")
public class ReservePreconstructController extends BaseController {

    @Autowired
    private ReservePreconstructService structService;


    @ModelAttribute
    public ReservePreconstruction get(@RequestParam(required = false) String id) {
        ReservePreconstruction entity = null;
        if (StringUtils.isNotBlank(id)) {
            entity = structService.get(id);
        }
        if (entity == null) {
            entity = new ReservePreconstruction();
        }
        return entity;
    }

    /**
     * 项目储备信息列表页面
     * @param reserveManagement
     * @param model
     * @return
     */
    @RequiresPermissions("cg:reservemanagementcenter:reservePreconstruct:list")
    @RequestMapping(value = "skipList")
    public String skipList(ReservePreconstruction reserveManagement, Model model) {
        model.addAttribute("reserveManagement", reserveManagement);
        return "modules/cg/reservemanagementcenter/reserveManage/reservePreconstruct/reserveManagementInformation";
    }



    /**
     * 项目储备信息数据列表
     * @param reserveManagement
     * @param request
     * @param response
     * @param model
     * @return
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:reservePreconstruct:list")
    @RequestMapping(value = "list")
    public Map<String, Object> basicData(ReservePreconstruction reserveManagement, HttpServletRequest request, HttpServletResponse response, Model model) {

        Page<ReservePreconstruction> page = structService.findPage(new Page<ReservePreconstruction>(request, response), reserveManagement);
        return getBootstrapData(page);
    }


    /**
     * 查看表单页面
     */
    @RequiresPermissions(value={"cg:reservemanagementcenter:reservePreconstruct:view"},logical= Logical.OR)
    @RequestMapping(value = "view")
    public String view(ReservePreconstruction reserveManagement, Model model) {
        model.addAttribute("reserveManagement", reserveManagement);
        return "modules/cg/reservemanagementcenter/reserveManage/reservePreconstruct/reserveManagementView";
    }


    /**
     * 专职人员保存表单
     * @param reserveManagement
     * @return
     */
    @ResponseBody
    @RequiresPermissions(value={"cg:reservemanagementcenter:reservePreconstruct:add"},logical=Logical.OR)
    @RequestMapping(value = "save")
    public AjaxJson save(ReservePreconstruction reserveManagement){
        AjaxJson j = new AjaxJson();
        /**
         * 后台hibernate-validation插件校验
         */
        String errMsg = beanValidator(reserveManagement);
        if (StringUtils.isNotBlank(errMsg)){
            j.setSuccess(false);
            j.setMsg(errMsg);
            return j;
        }
        //新增或编辑表单保存
        structService.save(reserveManagement);//保存
        j.setSuccess(true);
        j.setMsg("保存储备项目基本信息成功");
        return j;
    }


    /**
     * 专职人员修改表单
     * @param reserveManagement
     * @return
     */
    @ResponseBody
    @RequiresPermissions(value={"cg:reservemanagementcenter:reservePreconstruct:edit"},logical=Logical.OR)
    @RequestMapping(value = "update")
    public AjaxJson update(ReservePreconstruction reserveManagement){
        AjaxJson j = new AjaxJson();
        /**
         * 后台hibernate-validation插件校验
         */
        String errMsg = beanValidator(reserveManagement);
        if (StringUtils.isNotBlank(errMsg)){
            j.setSuccess(false);
            j.setMsg(errMsg);
            return j;
        }
        //新增或编辑表单保存
        structService.update(reserveManagement);
        j.setSuccess(true);
        j.setMsg("修改储备项目基本信息成功");
        return j;
    }

    /**
     * 导入Excel数据
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:reservePreconstruct:import")
    @RequestMapping(value = "import")
    public AjaxJson importBasic(@RequestParam("file") MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
        AjaxJson j = new AjaxJson();
        try {
            ImportUtil importUtil = new ImportUtil(file, 0, 0);
            //获取导入表格所有数据
            List<ReservePreconstruction> reserveManagementList = ReserveProcessProjectProUtil.getPreconstUtil(importUtil);
            //查询已有的参数数据 并分类返回已存在数据和未存在数据
            Map map = structService.getExistProjectList(reserveManagementList);
            //未存在数据
            List<ReservePreconstruction> withoutList = (List<ReservePreconstruction>) map.get("without");
            //已存在数据
            List<ReservePreconstruction> existList = (List<ReservePreconstruction>) map.get("exist");

            //对未存在的数据进行批量导入
            if(withoutList.size()>0){
                structService.insertList(withoutList);
            }

            //对已存在的数据进行修改
            if(existList.size()>0){
                for (ReservePreconstruction reserveManagement : existList) {
                    structService.update(reserveManagement);
                }
            }
            int i = withoutList.size() + existList.size();
            j.setMsg("已成功导入" + i + "条项目储备信息记录");

        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入项目储备信息失败！失败信息：" + e.getMessage());
        }
        return j;
    }


    //导出excel
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:reservePreconstruct:export")
    @RequestMapping(value = "export")
    public void exportExcel(ReservePreconstruction reserveManagement, HttpServletRequest request, HttpServletResponse response) {
        List<ReservePreconstruction> list = structService.exportFindList(reserveManagement);
        //生成序列号
        int i = 1;
        for (ReservePreconstruction s : list) {
            s.setId(String.valueOf(i));
            i++;
        }

        ReserveProcessProjectExportUtil util = ReserveProcessProjectExportUtil.getOne();
        XSSFWorkbook workbook = util.getWorkbook("preconstruct");
        util.exportPrecons(response,workbook,list);
    }


    /**
     * 专职人员增加，编辑表单页面
     * @param reserveManagement
     * @param model
     * @return
     */
    @RequiresPermissions(value={"cg:reservemanagementcenter:reservePreconstruct:add","cg:reservemanagementcenter:reservePreconstruct:edit"},logical= Logical.OR)
    @RequestMapping(value = "form")
    public String form(ReservePreconstruction reserveManagement, Model model) {
        model.addAttribute("reserveManagement", reserveManagement);
        //判断id是否为空 如果未空则为新增 否则为修改
        if(StringUtils.isBlank(reserveManagement.getId())){
            return "modules/cg/reservemanagementcenter/reserveManage/reservePreconstruct/reserveManagementForm";
        }
        return "modules/cg/reservemanagementcenter/reserveManage/reservePreconstruct/reserveManagementModifyByzz";
    }
}
