package com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.web;

import com.google.common.collect.Lists;
import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.entity.ReserveProcessProject;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.service.ReserveProcessProjectService;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.utils.ReserveProcessProjectExportUtil;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.utils.ReserveProcessProjectProUtil;
import com.jeeplus.modules.filing.basicInformation.entity.BasicInformation;
import com.jeeplus.modules.sg.managementcenter.materialproject.util.ImportUtil;
import com.jeeplus.modules.sg.managementcenter.reserveManagement.entity.ExportScheme;
import com.jeeplus.modules.sg.managementcenter.reserveManagement.service.ReserveManagementService;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**
 * 储备项目基本信息 controller
 * @author: 王强
 * @create: 2022-07-20 13:58
 **/
@Controller
@RequestMapping(value = "${adminPath}/cg/reservemanagementcenter/reserveManage")
public class ReserveProcessProjectController extends BaseController {

    @Autowired
    private ReserveManagementService service;

    @Autowired
    private ReserveProcessProjectService projectService;

    @ModelAttribute
    public ReserveProcessProject get(@RequestParam(required = false) String id) {
        ReserveProcessProject entity = null;
        if (StringUtils.isNotBlank(id)) {
            entity = projectService.get(id);
        }
        if (entity == null) {
            entity = new ReserveProcessProject();
        }
        return entity;
    }

    /**
     * 项目储备信息列表页面
     * @param reserveManagement
     * @param model
     * @return
     */
    @RequiresPermissions("cg:reservemanagementcenter:reserveManage:list")
    @RequestMapping(value = "skipList")
    public String skipList(ReserveProcessProject reserveManagement, Model model) {
        model.addAttribute("reserveManagement", reserveManagement);
        return "modules/cg/reservemanagementcenter/reserveManage/reserveManage/reserveManagementInformation";
    }


    /**
     * 方案交底结论列表页面
     * @param reserveManagement
     * @param model
     * @return
     */
    @RequiresPermissions("cg:reservemanagementcenter:disClosureReserveModule:list")
    @RequestMapping(value = "moduleList")
    public String moduleList(ReserveProcessProject reserveManagement, Model model) {
        model.addAttribute("reserveManagement", reserveManagement);
        return "modules/cg/reservemanagementcenter/reserveManage/disClosureReserveModuleList";
    }


    /**
     * 项目储备信息数据列表
     * @param reserveManagement
     * @param request
     * @param response
     * @param model
     * @return
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:reserveManage:list")
    @RequestMapping(value = "list")
    public Map<String, Object> basicData(ReserveProcessProject reserveManagement, HttpServletRequest request, HttpServletResponse response, Model model) {

        Page<ReserveProcessProject> page = projectService.findPage(new Page<ReserveProcessProject>(request, response), reserveManagement);
        return getBootstrapData(page);
    }

    /**
     * 方案交底结论数据列表
     * @param reserveManagement
     * @param request
     * @param response
     * @param model
     * @return
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:disClosureReserveModule:list")
    @RequestMapping(value = "moduleForm")
    public Map<String, Object> moduleData(ReserveProcessProject reserveManagement, HttpServletRequest request, HttpServletResponse response, Model model) {
        Page<ReserveProcessProject> page = projectService.findModulePage(new Page<ReserveProcessProject>(request, response), reserveManagement);
        return getBootstrapData(page);
    }


    /**
     * 专职人员增加，编辑表单页面
     * @param reserveManagement
     * @param model
     * @return
     */
    @RequiresPermissions(value={"cg:reservemanagementcenter:reserveManage:add","cg:reservemanagementcenter:reserveManage:edit"},logical= Logical.OR)
    @RequestMapping(value = "form")
    public String form(ReserveProcessProject reserveManagement, Model model) {
        model.addAttribute("reserveManagement", reserveManagement);
        //判断id是否为空 如果未空则为新增 否则为修改
        if(StringUtils.isBlank(reserveManagement.getId())){
            return "modules/cg/reservemanagementcenter/reserveManage/reserveManage/reserveManagementForm";
        }
        return "modules/cg/reservemanagementcenter/reserveManage/reserveManage/reserveManagementModifyByzz";
    }

    /**
     * 项目经理编辑表单页面
     * @param reserveManagement
     * @param model
     * @return
     */
    @RequiresPermissions(value={"cg:reservemanagementcenter:reserveManage:jlEdit"},logical= Logical.OR)
    @RequestMapping(value = "jlEdit")
    public String jlEdit(ReserveProcessProject reserveManagement, Model model) {
        model.addAttribute("reserveManagement", reserveManagement);
        return "modules/cg/reservemanagementcenter/reserveManage/reserveManage/reserveManagementModifyByjl";
    }

    /**
     * 查看表单页面
     */
    @RequiresPermissions(value={"cg:reservemanagementcenter:reserveManage:view"},logical= Logical.OR)
    @RequestMapping(value = "view")
    public String view(ReserveProcessProject reserveManagement, Model model) {
        model.addAttribute("reserveManagement", reserveManagement);
        return "modules/cg/reservemanagementcenter/reserveManage/reserveManage/reserveManagementView";
    }


    /**
     * 专职人员保存表单
     * @param reserveManagement
     * @return
     */
    @ResponseBody
    @RequiresPermissions(value={"cg:reservemanagementcenter:reserveManage:add"},logical=Logical.OR)
    @RequestMapping(value = "save")
    public AjaxJson save(ReserveProcessProject reserveManagement){
        AjaxJson j = new AjaxJson();
        /**
         * 后台hibernate-validation插件校验
         */
        String errMsg = beanValidator(reserveManagement);
        if (StringUtils.isNotBlank(errMsg)){
            j.setSuccess(false);
            j.setMsg(errMsg);
            return j;
        }
        //新增或编辑表单保存
        projectService.save(reserveManagement);//保存
        j.setSuccess(true);
        j.setMsg("保存储备项目基本信息成功");
        return j;
    }


    /**
     * 专职人员修改表单
     * @param reserveManagement
     * @return
     */
    @ResponseBody
    @RequiresPermissions(value={"cg:reservemanagementcenter:reserveManage:edit"},logical=Logical.OR)
    @RequestMapping(value = "update")
    public AjaxJson update(ReserveProcessProject reserveManagement){
        AjaxJson j = new AjaxJson();
        /**
         * 后台hibernate-validation插件校验
         */
        String errMsg = beanValidator(reserveManagement);
        if (StringUtils.isNotBlank(errMsg)){
            j.setSuccess(false);
            j.setMsg(errMsg);
            return j;
        }
        //新增或编辑表单保存
        projectService.update(reserveManagement);
        j.setSuccess(true);
        j.setMsg("修改储备项目基本信息成功");
        return j;
    }

    /**
     * 项目经理修改表单
     * @param reserveManagement
     * @return
     */
    @ResponseBody
    @RequiresPermissions(value={"cg:reservemanagementcenter:reserveManage:jlEdit"},logical=Logical.OR)
    @RequestMapping(value = "jlUpdate")
    public AjaxJson jlUpdate(ReserveProcessProject reserveManagement){
        AjaxJson j = new AjaxJson();

        //后台hibernate-validation插件校验
        String errMsg = beanValidator(reserveManagement);
        if (StringUtils.isNotBlank(errMsg)){
            j.setSuccess(false);
            j.setMsg(errMsg);
            return j;
        }
        projectService.updatejl(reserveManagement);
        j.setSuccess(true);
        j.setMsg("修改储备项目基本信息成功");
        return j;
    }

    /**
     * 删除表单
     */
    @ResponseBody
    @RequiresPermissions(value={"cg:reservemanagementcenter:reserveManage:del"},logical=Logical.OR)
    @RequestMapping(value = "deleteAll")
    public AjaxJson delete(String ids){
        AjaxJson j = new AjaxJson();
        String idArray[] =ids.split(",");
        for(String id : idArray){
            projectService.delete(projectService.get(id));
        }
        j.setMsg("删除储备项目基本信息成功");
        return j;
    }

    /**
     * 专职人员导入Excel数据
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:reserveManage:import")
    @RequestMapping(value = "import")
    public AjaxJson importBasic(@RequestParam("file") MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
        AjaxJson j = new AjaxJson();
        try {
            ImportUtil importUtil = new ImportUtil(file, 0, 0);
            //获取导入表格所有数据
            List<ReserveProcessProject> reserveManagementList = ReserveProcessProjectProUtil.getProjectUtil(importUtil);
            //查询已有的参数数据 并分类返回已存在数据和未存在数据
            Map map = projectService.getExistProjectList(reserveManagementList);
            //未存在数据
            List<ReserveProcessProject> withoutList = (List<ReserveProcessProject>) map.get("without");
            //已存在数据
            List<ReserveProcessProject> existList = (List<ReserveProcessProject>) map.get("exist");

            //对未存在的数据进行批量导入
            if(withoutList.size()>0){
                projectService.insertList(withoutList);
            }

            //对已存在的数据进行修改
            if(existList.size()>0){
                for (ReserveProcessProject reserveManagement : existList) {
                    projectService.update(reserveManagement);
                }
            }
            int i = withoutList.size() + existList.size();
            j.setMsg("已成功导入" + i + "条项目储备信息记录");

        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入项目储备信息失败！失败信息：" + e.getMessage());
        }
        return j;
    }

    /**
     * 经理导入Excel数据
     */
    /*@ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:reserveManage:jlImport")
    @RequestMapping(value = "jlImport")
    public AjaxJson jlImportBasic(@RequestParam("file") MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
        AjaxJson j = new AjaxJson();
        try {
            ImportExcel ei = new ImportExcel(file, 3, 0);
            List<ReserveProcessProject> list = ei.getDataList(ReserveProcessProject.class);
            for (ReserveProcessProject s :
                    list) {
                projectService.jlSave(s);
            }
            j.setMsg("已成功导入");

        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入项目储备信息失败！失败信息：" + e.getMessage());
        }
        return j;
    }*/

    //导出excel
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:reserveManage:export")
    @RequestMapping(value = "export")
    public void exportExcel(ReserveProcessProject reserveManagement, HttpServletRequest request, HttpServletResponse response) {
        List<ReserveProcessProject> list = projectService.exportFindList(reserveManagement);
        //生成序列号
        int i = 1;
        for (ReserveProcessProject s : list) {
            s.setId(String.valueOf(i));
            i++;
        }

        ReserveProcessProjectExportUtil util = ReserveProcessProjectExportUtil.getOne();
        XSSFWorkbook workbook = util.getWorkbook("projectReserveManagement");
        util.exportBasic(response,workbook,list);
    }


    /**
     * 下载导入基础信息数据模板
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:reserveManage:import")
    @RequestMapping(value = "template")
    public AjaxJson importTemplate(HttpServletResponse response) {
        AjaxJson j = new AjaxJson();
        try {
            String fileName = "储备项目基础信息导入模板.xlsx";
            List<BasicInformation> list = Lists.newArrayList();
            new ExportExcel("储备项目基础信息", BasicInformation.class, 1).setDataList(list).write(response, fileName).dispose();
            return null;
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg( "导入模板下载失败！失败信息："+e.getMessage());
        }
        return j;
    }


    /**
     * 下载导入基础信息数据模板
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:reserveManage:import")
    @RequestMapping(value = "schemeTemplate")
    public AjaxJson importSchemeTemplate(HttpServletResponse response) {
        AjaxJson j = new AjaxJson();
        try {
            String fileName = "方案交底结论导入模板.xlsx";
            List<BasicInformation> list = Lists.newArrayList();
            new ExportExcel("方案交底结论", ExportScheme.class, 1).setDataList(list).write(response, fileName).dispose();
            return null;
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg( "导入模板下载失败！失败信息："+e.getMessage());
        }
        return j;
    }
}
