package com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.web;

import com.google.common.collect.Maps;
import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.act.service.ActProcessService;
import com.jeeplus.modules.act.service.ActTaskService;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.entity.ReserveProcessProject;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.entity.ReserveSummary;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.entity.ReserveSummaryProcess;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.service.ReserveProcessProjectService;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.service.ReserveSummaryProcessService;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.service.SummaryStatisticService;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.utils.ReserveProcessProjectExportUtil;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.utils.ReserveProcessProjectProUtil;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.service.BudgetStatsService;
import com.jeeplus.modules.sg.managementcenter.materialproject.util.ImportUtil;
import com.jeeplus.modules.sg.managementcenter.reserveManagement.service.PersonnelTurnoverService;
import com.jeeplus.modules.sys.utils.UserUtils;
import org.activiti.engine.repository.ProcessDefinition;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**
 * 储备项目交底信息汇总统计 controller
 * @author: 王强
 * @create: 2022-07-27 11:03
 **/
@Controller
@RequestMapping(value = "${adminPath}/cg/reservemanagementcenter/summaryStatistics")
public class SummaryStatisticController extends BaseController {

    @Autowired
    private PersonnelTurnoverService service;

    @Autowired
    private SummaryStatisticService statisticService;

    @Autowired
    private ReserveProcessProjectService projectService;

    @Autowired
    private ActProcessService actProcessService;

    @Autowired
    private ActTaskService actTaskService;

    @Autowired
    private ReserveSummaryProcessService reserveProcessService;

    @Autowired
    private BudgetStatsService statsService;


    @ModelAttribute
    public ReserveSummary get(@RequestParam(required = false) String id) {
        ReserveSummary entity = null;
        if (StringUtils.isNotBlank(id)) {
            entity = statisticService.get(id);
        }
        if (entity == null) {
            entity = new ReserveSummary();
        }
        return entity;
    }

    /**
     * 项目储备信息列表页面
     * @param reserveManagement
     * @param model
     * @return
     */
    @RequiresPermissions("cg:reservemanagementcenter:summaryStatistics:list")
    @RequestMapping(value = "skipList")
    public String skipList(ReserveSummary reserveManagement, Model model) {
        model.addAttribute("reserveManagement", reserveManagement);
        return "modules/cg/reservemanagementcenter/reserveManage/summaryStatistics/reserveManagementInformation";
    }



    /**
     * 项目储备信息数据列表
     * @param reserveManagement
     * @param request
     * @param response
     * @param model
     * @return
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:summaryStatistics:list")
    @RequestMapping(value = "list")
    public Map<String, Object> basicData(ReserveSummary reserveManagement, HttpServletRequest request, HttpServletResponse response, Model model) {

        Page<ReserveSummary> page = statisticService.findPage(new Page<ReserveSummary>(request, response), reserveManagement);
        return getBootstrapData(page);
    }


    /**
     * 查看表单页面
     */
    @RequiresPermissions(value={"cg:reservemanagementcenter:summaryStatistics:view"},logical= Logical.OR)
    @RequestMapping(value = "view")
    public String view(ReserveSummary reserveManagement, Model model) {
        model.addAttribute("reserveManagement", reserveManagement);
        return "modules/cg/reservemanagementcenter/reserveManage/summaryStatistics/reserveManagementView";
    }


    /**
     * 专职人员保存表单
     * @param reserveManagement
     * @return
     */
    @ResponseBody
    @RequiresPermissions(value={"cg:reservemanagementcenter:summaryStatistics:add"},logical=Logical.OR)
    @RequestMapping(value = "save")
    public AjaxJson save(ReserveSummary reserveManagement){
        AjaxJson j = new AjaxJson();
        /**
         * 后台hibernate-validation插件校验
         */
        String errMsg = beanValidator(reserveManagement);
        if (StringUtils.isNotBlank(errMsg)){
            j.setSuccess(false);
            j.setMsg(errMsg);
            return j;
        }
        //新增或编辑表单保存
        statisticService.save(reserveManagement);//保存
        j.setSuccess(true);
        j.setMsg("保存储备项目基本信息成功");
        return j;
    }


    /**
     * 专职人员修改表单
     * @param reserveManagement
     * @return
     */
    @ResponseBody
    @RequiresPermissions(value={"cg:reservemanagementcenter:summaryStatistics:edit"},logical=Logical.OR)
    @RequestMapping(value = "update")
    public AjaxJson update(ReserveSummary reserveManagement){
        AjaxJson j = new AjaxJson();
        /**
         * 后台hibernate-validation插件校验
         */
        String errMsg = beanValidator(reserveManagement);
        if (StringUtils.isNotBlank(errMsg)){
            j.setSuccess(false);
            j.setMsg(errMsg);
            return j;
        }
        //新增或编辑表单保存
        statisticService.update(reserveManagement);
        j.setSuccess(true);
        j.setMsg("修改储备项目基本信息成功");
        return j;
    }

    /**
     * 导入Excel数据
     */
   /* @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:summaryStatistics:import")
    @RequestMapping(value = "import")
    public AjaxJson importBasic(@RequestParam("file") MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
        AjaxJson j = new AjaxJson();
        try {
            ImportUtil importUtil = new ImportUtil(file, 0, 0);
            //获取导入表格所有数据
            List<ReserveSummary> reserveManagementList = ReserveProcessProjectProUtil.getSummaryUtil(importUtil);
            //查询已有的参数数据 并分类返回已存在数据和未存在数据
            Map map = statisticService.getExistProjectList(reserveManagementList);
            //未存在数据
            List<ReserveSummary> withoutList = (List<ReserveSummary>) map.get("without");
            //已存在数据
            List<ReserveSummary> existList = (List<ReserveSummary>) map.get("exist");

            //对未存在的数据进行批量导入
            if(withoutList.size()>0){
                statisticService.insertList(withoutList);
            }

            //对已存在的数据进行修改
            if(existList.size()>0){
                for (ReserveSummary reserveManagement : existList) {
                    statisticService.update(reserveManagement);
                }
            }
            int i = withoutList.size() + existList.size();
            j.setMsg("已成功导入" + i + "条项目储备信息记录");

        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入项目储备信息失败！失败信息：" + e.getMessage());
        }
        return j;
    }
*/

    /**
     * 导入Excel数据
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:summaryStatistics:import")
    @RequestMapping(value = "import")
    public AjaxJson importBasic(@RequestParam("file") MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
        AjaxJson j = new AjaxJson();
        try {
            ImportUtil importUtil = new ImportUtil(file, 0, 0);
            //获取导入表格所有数据
            List<ReserveSummary> reserveManagementList = ReserveProcessProjectProUtil.getSummaryUtil(importUtil);
            //查询已有的参数数据 并分类返回已存在数据和未存在数据
            Map map = statisticService.getExistProjectList(reserveManagementList);
            //未存在数据
            List<ReserveSummary> withoutList = (List<ReserveSummary>) map.get("without");
            //已存在数据
            List<ReserveSummary> existList = (List<ReserveSummary>) map.get("exist");

            if (withoutList.size()>0){
                for (ReserveSummary s : withoutList){
                    /*s.setOpenProcessStatus("1");
                    System.out.println("ReserveSummary：" + s);*/
                    statisticService.insert(s);

                    if (s.getIncludeReserve().equals("是")){

                        ReserveSummaryProcess reserveProcess = new ReserveSummaryProcess();
                        reserveProcess.preInsert();
                        reserveProcess.setProjectReserveId(s.getProjectReserveId());
                        //更新
                        reserveProcessService.insert(reserveProcess);
                        //发送流程，更改项目状态
                        Map<String, Object> vars = Maps.newHashMap();

                        ReserveProcessProject processProject = projectService.queryByProjectId(s.getProjectReserveId());
                        //审核代理人
                        vars.put("manager", UserUtils.getByUserName(processProject.getDesigner()).getLoginName());
                        String procDefId = projectService.findDefId("cblc");
                        ProcessDefinition p = actProcessService.getProcessDefinition(procDefId);
                        String title = s.getCurrentUser().getName() + "在" + DateUtils.getDateTime() + "发起" + p.getName();
                        actTaskService.startProcess(p.getKey(), "cg_personnel_activiti", reserveProcess.getId(), title, vars);
                        /*int count = statsService.selectCount(reserveProcess.getProjectReserveId());
                        if (count>0){
                            statsService.updateStats("设计单位在线开甲供材",null,null,reserveProcess.getProjectReserveId());
                        }*/
                    }
                }
            }

            //对已存在的数据进行修改
            if(existList.size()>0){
                for (ReserveSummary reserveManagement : existList) {
                    //去掉之前的流程，重新开始流程
                    reserveProcessService.deleteAct(reserveManagement.getProjectReserveId());
                    statisticService.update(reserveManagement);
                    if (reserveManagement.getIncludeReserve().equals("是")){
                        ReserveSummaryProcess reserveProcess = new ReserveSummaryProcess();
                        reserveProcess.preInsert();
                        reserveProcess.setProjectReserveId(reserveManagement.getProjectReserveId());
                        //更新
                        reserveProcessService.insert(reserveProcess);
                        //发送流程，更改项目状态
                        Map<String, Object> vars = Maps.newHashMap();

                        ReserveProcessProject processProject = projectService.queryByProjectId(reserveManagement.getProjectReserveId());
                        vars.put("manager", UserUtils.getByUserName(processProject.getDesigner()).getLoginName());
                        String procDefId = projectService.findDefId("cblc");
                        ProcessDefinition p = actProcessService.getProcessDefinition(procDefId);
                        String title = reserveManagement.getCurrentUser().getName() + "在" + DateUtils.getDateTime() + "发起" + p.getName();
                        actTaskService.startProcessNew(p.getKey(), "cg_personnel_activiti", reserveProcess.getId(), title, vars);
                    }
                }
            }

            int i = withoutList.size() + existList.size();
            j.setMsg("已成功导入" + i + "条项目储备信息记录");
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入项目规模信息失败！失败信息：" + e.getMessage());
        }
        return j;
    }


    //导出excel
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:summaryStatistics:export")
    @RequestMapping(value = "export")
    public void exportExcel(ReserveSummary reserveManagement, HttpServletRequest request, HttpServletResponse response) {
        List<ReserveSummary> list = statisticService.exportFindList(reserveManagement);
        //生成序列号
        int i = 1;
        for (ReserveSummary s : list) {
            s.setId(String.valueOf(i));
            i++;
        }

        ReserveProcessProjectExportUtil util = ReserveProcessProjectExportUtil.getOne();
        XSSFWorkbook workbook = util.getWorkbook("summaryStatistics");
        util.exportSummary(response,workbook,list);
    }


    /**
     * 专职人员增加，编辑表单页面
     * @param reserveManagement
     * @param model
     * @return
     */
    @RequiresPermissions(value={"cg:reservemanagementcenter:summaryStatistics:add","cg:reservemanagementcenter:summaryStatistics:edit"},logical= Logical.OR)
    @RequestMapping(value = "form")
    public String form(ReserveSummary reserveManagement, Model model) {
        model.addAttribute("reserveManagement", reserveManagement);
        //判断id是否为空 如果未空则为新增 否则为修改
        if(StringUtils.isBlank(reserveManagement.getId())){
            return "modules/cg/reservemanagementcenter/reserveManage/summaryStatistics/reserveManagementForm";
        }
        return "modules/cg/reservemanagementcenter/reserveManage/summaryStatistics/reserveManagementModifyByzz";
    }

}
