package com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.util;

import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.BudgetStats;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.ConstructeScale;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.ProgressStats;
import com.jeeplus.modules.sg.managementcenter.materialproject.util.ImportUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Row;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

/**
 * @author: 王强
 * @create: 2022-08-16 15:01
 **/
public class ExcelImportUtil {

    /**
     * 获取储备项目概预算统计表表格数据
     *
     * @param importUtil
     * @return List<BudgetStats> 导入表格数据
     */
    public static List<BudgetStats> getBudgetUtil(ImportUtil importUtil) throws ParseException {
        int lastRow = importUtil.getLastDataRowNum();
        List<BudgetStats> list = new ArrayList<BudgetStats>();
        BudgetStats budgetStats = null;

        for(int i=2; i<=lastRow;i++) {
            budgetStats = new BudgetStats();
            Row row = importUtil.getRow(i);

            String reserveProjectName = (String) importUtil.getCellValue(row, 1);
            //获取项目储备编号
            String projectReserveId = (String) importUtil.getCellValue(row, 10);
            if (StringUtils.isNotBlank(reserveProjectName)) {
                if (StringUtils.isNotBlank(projectReserveId)) {

                    budgetStats.setProjectReserveId((String) importUtil.getCellValue(row, 10));
                    budgetStats.setPretrialEstimateAmount((String) importUtil.getCellValue(row, 22));
                    budgetStats.setPretrialEstimateAmountExcludeTax((String) importUtil.getCellValue(row, 23));
                    budgetStats.setPretrialDeductibleAmount((String) importUtil.getCellValue(row, 24));
                    budgetStats.setPretrialConstructeConsts((String) importUtil.getCellValue(row, 25));
                    budgetStats.setPretrialMaterialAmount((String) importUtil.getCellValue(row, 26));
                    budgetStats.setPretrialOtherExpanses((String) importUtil.getCellValue(row, 27));
                    budgetStats.setPretrialPreliminaryConstructeFee((String) importUtil.getCellValue(row, 28));
                    budgetStats.setPretrialSurveyFee((String) importUtil.getCellValue(row, 29));
                    budgetStats.setPretrialDesignFee((String) importUtil.getCellValue(row, 30));
                    budgetStats.setPretrialSupervisionFee((String) importUtil.getCellValue(row, 31));
                    budgetStats.setAfterTrialEstimateAmount((String) importUtil.getCellValue(row, 32));
                    budgetStats.setAfterTrialEstimateAmountExcludeTax((String) importUtil.getCellValue(row, 33));
                    budgetStats.setAfterTrialDeductibleAmount((String) importUtil.getCellValue(row, 34));
                    budgetStats.setAfterTrialConstructeConsts((String) importUtil.getCellValue(row, 35));
                    budgetStats.setAfterTrialMaterialAmount((String) importUtil.getCellValue(row, 36));
                    budgetStats.setAfterTrialOtherExpanses((String) importUtil.getCellValue(row, 37));
                    budgetStats.setAfterTrialPreliminaryConstructeFee((String) importUtil.getCellValue(row, 38));
                    budgetStats.setAfterTrialSurveyFee((String) importUtil.getCellValue(row, 39));
                    budgetStats.setAfterTrialDesignFee((String) importUtil.getCellValue(row, 40));
                    budgetStats.setAfterTrialSupervisionFee((String) importUtil.getCellValue(row, 41));

                    list.add(budgetStats);
                }

            }
        }
        return list;
    }

    /**
     * 获取储备进度统计表表格数据
     *
     * @param importUtil
     * @return List<BudgetStats> 导入表格数据
     */
    public static List<ProgressStats> getProgressUtil(ImportUtil importUtil) throws ParseException {
        int lastRow = importUtil.getLastDataRowNum();
        List<ProgressStats> list = new ArrayList<ProgressStats>();
        ProgressStats progressStats = null;

        for (int i = 3; i <= lastRow; i++) {
            progressStats = new ProgressStats();
            Row row = importUtil.getRow(i);

            String reserveProjectName = (String) importUtil.getCellValue(row, 1);
            //获取项目储备编号
            String projectReserveId = (String) importUtil.getCellValue(row, 0);
            if (StringUtils.isNotBlank(reserveProjectName)) {
                if (StringUtils.isNotBlank(projectReserveId)) {
                    progressStats.setProjectReserveId((String) importUtil.getCellValue(row, 0));//项目储备编号
                    progressStats.setInspectDepartmentReview((String) importUtil.getCellValue(row, 9));//是否通过运检部评审
                    progressStats.setInspectDepartmentReviewTime((String) importUtil.getCellValue(row, 10));//通过运检部评审时间
                    progressStats.setReleaseReserveTime((String) importUtil.getCellValue(row, 11));//运检部下达储备时间
                    progressStats.setOnSiteConfess((String) importUtil.getCellValue(row, 15));//是否进行现场交底
                    progressStats.setConfessCompleteTime((String) importUtil.getCellValue(row, 16));//交底完成时间
                    progressStats.setProvincialCompanyReview((String) importUtil.getCellValue(row, 30));//省公司评审-是否参加省公司评审
                    progressStats.setProvincialCompanyReviewBatch((String) importUtil.getCellValue(row, 31));//省公司评审-省公司评审批次
                    progressStats.setPassProvincialCompanyReview((String) importUtil.getCellValue(row, 32));//省公司评审-是否通过省公司评审
                    progressStats.setCauseAnalysis((String) importUtil.getCellValue(row, 33));//省公司评审-未通过原因分析
                    progressStats.setDesignChange((String) importUtil.getCellValue(row, 34));//省公司评审-是否需设计变更
                    progressStats.setDrawingReview((String) importUtil.getCellValue(row, 35));//图纸联合审查-是否进行图纸审查
                    progressStats.setReviewDate((String) importUtil.getCellValue(row, 36));//图纸联合审查-审图日期
                    progressStats.setPassDrawingReview((String) importUtil.getCellValue(row, 37));//图纸联合审查-是否通过图纸审查
                    progressStats.setReviewComment((String) importUtil.getCellValue(row, 38));//图纸联合审查-未通过审图意见
                    progressStats.setApprovedDate((String) importUtil.getCellValue(row, 39));//图纸联合审查-通过审核日期
                    progressStats.setJointReviewDesignChange((String) importUtil.getCellValue(row, 40));//图纸联合审查-是否需设计变更
                    progressStats.setFormalDrawing((String) importUtil.getCellValue(row, 43));//是否打印正式图纸
                    progressStats.setDrawingDate((String) importUtil.getCellValue(row, 44));//通知出图日期
                    progressStats.setTransferProjectCenterDate((String) importUtil.getCellValue(row, 45));//正式图纸移交项目中心日期
                    progressStats.setTransferToConstructionUnitDate((String) importUtil.getCellValue(row, 46));//正式图纸移交施工单位日期

                    list.add(progressStats);
                }
            }
        }
        return list;
    }

    /**
     * 获取储备项目建设规模统计表表格数据
     *
     * @param importUtil
     * @return List<BudgetStats> 导入表格数据
     */
    public static List<ConstructeScale> getConstructeUtil(ImportUtil importUtil) throws ParseException {
        int lastRow = importUtil.getLastDataRowNum();
        List<ConstructeScale> list = new ArrayList<ConstructeScale>();
        ConstructeScale constructeScale = null;

        for (int i = 3; i <= lastRow; i++) {
            constructeScale = new ConstructeScale();
            Row row = importUtil.getRow(i);

            String reserveProjectName = (String) importUtil.getCellValue(row, 7);
            //获取项目储备编号
            String projectReserveId = (String) importUtil.getCellValue(row, 6);
            if (StringUtils.isNotBlank(reserveProjectName)) {
                if (StringUtils.isNotBlank(projectReserveId)) {
                    constructeScale.setProjectReserveId((String) importUtil.getCellValue(row, 6));//项目储备编号
                    constructeScale.setConstructePurpose((String) importUtil.getCellValue(row, 9));//建设目的
                    constructeScale.setFactoryPrefab((String) importUtil.getCellValue(row, 32));//土建工厂化预制
                    constructeScale.setAncillaryFacility((String) importUtil.getCellValue(row, 33));//附属设施
                    constructeScale.setConstructionWorkMethod((String) importUtil.getCellValue(row, 34));//施工作业方式
                    constructeScale.setWorkWithoutInterruption((String) importUtil.getCellValue(row, 35));//不停电作业
                    String number = (String)importUtil.getCellValue(row, 36) == ""? "0" : (String)importUtil.getCellValue(row, 36);
                    int num = Integer.parseInt(number);
                    constructeScale.setPowerFailureHouseNum(num);//停电时户数
                    constructeScale.setPeakSummerProject((String) importUtil.getCellValue(row, 38));//是否迎峰度夏项目
                    constructeScale.setProjectReview((String) importUtil.getCellValue(row, 39));//通过第几批项目评审

                    list.add(constructeScale);
                }
            }
        }
        return list;
    }
}
