package com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.web;

import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.service.SummaryStatisticService;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.BudgetStats;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.History;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.service.BudgetStatsService;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.util.ExcelExportUtil;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.util.ExcelImportUtil;
import com.jeeplus.modules.sg.managementcenter.materialproject.util.ImportUtil;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 储备项目概预算统计表
 * @author: 王强
 * @create: 2022-08-16 13:47
 **/
@Controller
@RequestMapping(value = "${adminPath}/cg/reservemanagementcenter/budgetStats")
public class BudgetStatsController extends BaseController {

    @Autowired
    private BudgetStatsService statsService;

    @Autowired
    private SummaryStatisticService statisticService;

    @ModelAttribute
    public BudgetStats get(@RequestParam(required = false) String id) {
        BudgetStats entity = null;
        if (StringUtils.isNotBlank(id)) {
            entity = statsService.get(id);
        }
        if (entity == null) {
            entity = new BudgetStats();
        }
        return entity;
    }

    /**
     * 储备项目概预算列表页面
     * @param budgetStats
     * @param model
     * @return
     */
    @RequiresPermissions("cg:reservemanagementcenter:budgetStats:list")
    @RequestMapping(value = "budgetList")
    public String budgetList(BudgetStats budgetStats, Model model) {
        model.addAttribute("budgetStats", budgetStats);
        return "modules/cg/statisticalTable/budgetStats/budgetStats";
    }

    /**
     * 储备项目概预算数据列表
     * @param budgetStats
     * @param request
     * @param response
     * @param model
     * @return
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:budgetStats:list")
    @RequestMapping(value = "list")
    public Map<String, Object> basicData(BudgetStats budgetStats, HttpServletRequest request, HttpServletResponse response, Model model) {

        Page<BudgetStats> page = statsService.findPage(new Page<BudgetStats>(request, response), budgetStats);
        List<BudgetStats> list = page.getList();
        List<BudgetStats> statsList = new ArrayList<>();
        BudgetStats stats = null;
        for (int i=0;i<list.size();i++){
            stats = list.get(i);
            //根据projectReserveId查出流程表的proc_ins_id
            String procInsId = statsService.getByProjectReserveId(stats.getProjectReserveId());
            //根据procInsId查出历史流程名称和时间
            List<History> histories = statsService.getHistoryByExecuteId(procInsId);
            //当size>0时待办有历史流程，
            if (histories.size()>0){
                //根据proc_ins_id去运行时任务节点表（act_ru_task）中查出当前流程名称
                String name = statsService.getByExecuteId(procInsId);
                //设置项目进度
                stats.setReserveProgress(name);

                //设置是否完成概算修改
                //第一种情况，修改中 先在历史表中查出技经人员审预算相关的信息
                List<History> his = statsService.getHistoryByActName(procInsId,"技经人员审预算");
                if (his.size()>0){
                    //根据他的完成时间来找下一个节点的信息
                    String strTime = his.get(0).getEndTime();
                    History history = statsService.getHistoryByTime(strTime,procInsId);
                    if (history == null){
                        stats.setBudgetRevision("无需修改");
                        stats.setCompleteTime(strTime);
                    }else {
                        //修改中的状态
                        if (history.getName().equals("设计单位查看错误原因")){
                            List<History> historyByActName = statsService.getHistoryByActName(procInsId,"设计单位查看错误原因");
                            String endTime = historyByActName.get(0).getEndTime();
                            if (endTime == null){
                                stats.setBudgetRevision("修改中");
                                stats.setCompleteTime(strTime);
                            }else {
                                //未完成状态
                                stats.setBudgetRevision("未完成");
                                stats.setCompleteTime(strTime);
                            }
                        }
                    }

                }
            }else {
                stats.setReserveProgress("");
            }
            statsList.add(stats);
        }
        page.setList(statsList);
        return getBootstrapData(page);
    }

    /**
     *  导入Excel数据
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:budgetStats:import")
    @RequestMapping(value = "import")
    public AjaxJson importBasic(@RequestParam("file") MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
        AjaxJson j = new AjaxJson();
        try {
            ImportUtil importUtil = new ImportUtil(file, 0, 0);
            //获取导入表格所有数据
            List<BudgetStats> budgetStats = ExcelImportUtil.getBudgetUtil(importUtil);
            statsService.insertList(budgetStats);
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入储备项目概预算信息失败！失败信息：" + e.getMessage());
        }
        return j;
    }

    /**
     * 删除表单
     */
    @ResponseBody
    @RequiresPermissions(value={"cg:reservemanagementcenter:budgetStats:del"},logical= Logical.OR)
    @RequestMapping(value = "deleteAll")
    public AjaxJson delete(String ids){
        AjaxJson j = new AjaxJson();
        String idArray[] =ids.split(",");
        for(String id : idArray){
            statsService.delete(statsService.get(id));
        }
        j.setMsg("删除储备项目概预算信息成功");
        return j;
    }

    //导出excel
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:budgetStats:export")
    @RequestMapping(value = "export")
    public void exportExcel(BudgetStats budgetStats, HttpServletRequest request, HttpServletResponse response) {
        List<BudgetStats> list = statsService.exportFindList(budgetStats);

        List<BudgetStats> statsList = new ArrayList<>();
        BudgetStats stats = null;
        for (int i=0;i<list.size();i++){
            stats = list.get(i);
            //根据projectReserveId查出流程表的proc_ins_id
            String procInsId = statsService.getByProjectReserveId(stats.getProjectReserveId());
            //根据procInsId查出历史流程名称和时间
            List<History> histories = statsService.getHistoryByExecuteId(procInsId);
            //当size>0时待办有历史流程，
            if (histories.size()>0){
                //根据proc_ins_id去运行时任务节点表（act_ru_task）中查出当前流程名称
                String name = statsService.getByExecuteId(procInsId);
                //设置项目进度
                stats.setReserveProgress(name);
                //设置是否完成概算修改
                //第一种情况，修改中 先在历史表中查出技经人员审预算相关的信息
                List<History> his = statsService.getHistoryByActName(procInsId,"技经人员审预算");
                if (his.size()>0){
                    //根据他的完成时间来找下一个节点的信息
                    String strTime = his.get(0).getEndTime();
                    History history = statsService.getHistoryByTime(strTime,procInsId);
                    if (history == null){
                        stats.setBudgetRevision("无需修改");
                        stats.setCompleteTime(strTime);
                    }else {
                        //修改中的状态
                        if (history.getName().equals("设计单位查看错误原因")){
                            List<History> historyByActName = statsService.getHistoryByActName(procInsId,"设计单位查看错误原因");
                            String endTime = historyByActName.get(0).getEndTime();
                            if (endTime == null){
                                stats.setBudgetRevision("修改中");
                                stats.setCompleteTime(strTime);
                            }else {
                                //未完成状态
                                stats.setBudgetRevision("未完成");
                                stats.setCompleteTime(strTime);
                            }
                        }
                    }

                }
            }else {
                stats.setReserveProgress("");
            }
            statsList.add(stats);
        }
        //生成序列号
        int i = 1;
        for (BudgetStats s : statsList) {
            s.setId(String.valueOf(i));
            i++;
        }

        ExcelExportUtil util = ExcelExportUtil.getOne();
        XSSFWorkbook workbook = util.getWorkbook("budgetStats");
        util.exportBasic(response,workbook,statsList);
    }
}
