package com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.web;

import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.MaterialData;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.StandardDetail;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.ConstructeScale;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.History;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.service.ConstructeScaleService;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.util.ExcelExportUtil;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.util.ExcelImportUtil;
import com.jeeplus.modules.sg.managementcenter.materialproject.util.ImportUtil;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 储备项目建设规模统计表
 * @author: 王强
 * @create: 2022-08-19 14:17
 **/
@Controller
@RequestMapping(value = "${adminPath}/cg/reservemanagementcenter/constructeScale")
public class ConstructeScaleController extends BaseController {

    @Autowired
    private ConstructeScaleService scaleService;

    @ModelAttribute
    public ConstructeScale get(@RequestParam(required = false) String id) {
        ConstructeScale entity = null;
        if (StringUtils.isNotBlank(id)) {
            entity = scaleService.get(id);
        }
        if (entity == null) {
            entity = new ConstructeScale();
        }
        return entity;
    }

    /**
     * 删除表单
     */
    @ResponseBody
    @RequiresPermissions(value={"cg:reservemanagementcenter:constructeScale:del"},logical= Logical.OR)
    @RequestMapping(value = "deleteAll")
    public AjaxJson delete(String ids){
        AjaxJson j = new AjaxJson();
        String idArray[] =ids.split(",");
        for(String id : idArray){
            scaleService.delete(scaleService.get(id));
        }
        j.setMsg("删除储备进度信息成功");
        return j;
    }

    /**
     * 储备项目建设规模统计列表页面
     * @param constructeScale
     * @param model
     * @return
     */
    @RequiresPermissions("cg:reservemanagementcenter:constructeScale:list")
    @RequestMapping(value = "scaleList")
    public String scaleList(ConstructeScale constructeScale, Model model) {
        model.addAttribute("constructeScale", constructeScale);
        return "modules/cg/statisticalTable/constructeScale/constructeScale";
    }

    /**
     *储备项目建设规模统计数据列表
     * @param constructeScale
     * @param request
     * @param response
     * @param model
     * @return
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:constructeScale:list")
    @RequestMapping(value = "list")
    public Map<String, Object> basicData(ConstructeScale constructeScale, HttpServletRequest request, HttpServletResponse response, Model model) {

        Page<ConstructeScale> page = scaleService.findPage(new Page<ConstructeScale>(request, response), constructeScale);
        //获取到建设规模统计数据
        List<ConstructeScale> scaleList = page.getList();
        //创建一个新的list来接收处理过后的数据
        List<ConstructeScale> list = new ArrayList<>();
        ConstructeScale scale = null;
        //赋值操作
        for (int i=0;i<scaleList.size();i++){
            scale = scaleList.get(i);
            //根据projectReserveId查出流程表的proc_ins_id
            String procInsId = scaleService.getByProjectReserveId(scale.getProjectReserveId());
            //根据proc_ins_id去运行时任务节点表（act_ru_task）中查出当前流程名称
            String name = scaleService.getByExecuteId(procInsId);
            //根据procInsId查出历史流程名称和时间
            List<History> histories = scaleService.getHistoryByExecuteId(procInsId);
            //当size>0时待办有历史流程，
            if (histories.size()>0){
                //设置项目进度
                scale.setReserveProgress(name);
                scale.setCityCompany("国网镇江供电公司");
                //根据相关规则设置物料信息
                //根物料规范的数据比对，判断在线开甲供材属于哪一类型的数据
                //先从物料详细数据中查出规范数据
                List<StandardDetail> detailList = scaleService.getStandardDetail();
                //在从在线开甲供材的表中查出所有数据
                List<MaterialData> materialDataList = scaleService.getMaterialData(scale.getProjectName());
                //进行对比
                if (materialDataList.size()>0){
                    double kgSum = 0;
                    double xlSum = 0;
                    double dlSum = 0;
                    double fzxSum = 0;
                    double hwxSum = 0;
                    double hwsSum = 0;
                    double zskgSum = 0;
                    double pdSum = 0;
                    double pdsSum = 0;
                    double xbSum = 0;
                    double xbkvaSum = 0;
                    double pdkvaSum = 0;
                    double pdkSum = 0;
                    double dyxlSum = 0;
                    double dydlSum = 0;
                    double dlfzxSum = 0;
                    double dlfzgSum = 0;
                    double pdftuSum = 0;
                    double pddtuSum = 0;
                    double glSum = 0;
                    double dltdSum = 0;
                    String num = "";
                    for (int j=0;j<materialDataList.size();j++){

                        for (int k=0;k<detailList.size();k++){

                            //通过物料编码和物料描述来进行对比
                            if (detailList.get(k).getMaterialCode().equals(materialDataList.get(j).getMaterialCode())
                                    && detailList.get(k).getMaterialDescription().equals(materialDataList.get(j).getMaterialName())){
                                if (detailList.get(k).getProjectCategory().equals("开关柜")){
                                    kgSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    scale.setSwitchCabinet(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("架空线路")){
                                    xlSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                   num = xlSum + "";
                                    scale.setOverheadLine(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("电力电缆")){
                                    dlSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = dlSum + "";
                                    scale.setPowerCable(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("电缆分支箱")){
                                    fzxSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = fzxSum + "";
                                    scale.setCableBranchBox(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("户外环网箱")){
                                    hwxSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = hwxSum + "";
                                    scale.setOutdoorRingCage(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("环网箱/室")){
                                    hwsSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = hwsSum + "";
                                    scale.setRingRoom(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("柱上开关")){
                                    zskgSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = zskgSum + "";
                                    scale.setOnColumnSwitch(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("配电室kva")){
                                    pdSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = pdSum + "";
                                    scale.setSwitchRoomKVA(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("配电室")){
                                    pdsSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = pdsSum + "";
                                    scale.setSwitchRoom(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("箱变kva")){
                                    xbkvaSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = xbkvaSum + "";
                                    scale.setBoxChangeKVA(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("箱变")){
                                    xbSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = xbSum + "";
                                    scale.setBoxChange(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("配电变压器kva")){
                                    pdkvaSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = pdkvaSum + "";
                                    scale.setDistributionTransformerKVA(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("配电变压器")){
                                    pdkSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = pdkSum + "";
                                    scale.setDistributionTransformer(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("低压线路")){
                                    dyxlSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = dyxlSum + "";
                                    scale.setLowVoltageLine(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("低压电缆")){
                                    dydlSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = dydlSum + "";
                                    scale.setLowVoltageCable(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("低压电缆分支箱")){
                                    dlfzxSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = dlfzxSum + "";
                                    scale.setLowVoltageCableBox(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("低压配电箱（柜）")){
                                    dlfzgSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = dlfzgSum + "";
                                    scale.setLowVoltageCabinet(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("配电自动化终端（FTU）")){
                                    pdftuSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = pdftuSum + "";
                                    scale.setDistributeAutoMateTerminalFTU(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("配电自动化终端（DTU）")){
                                    pddtuSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = pddtuSum + "";
                                    scale.setDistributeAutoMateTerminalDTU(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("光缆")){
                                    glSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = glSum + "";
                                    scale.setOpticalCable(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("电缆通道")){
                                    dltdSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = dltdSum + "";
                                    scale.setCablePassage(num);
                                }
                            }
                        }
                    }
                }
            }else {
                //设置项目进度
                scale.setReserveProgress("");
            }
            list.add(scale);
        }
        System.out.println("list：" + list);
        page.setList(list);
        return getBootstrapData(page);
    }

    /**
     *  导入Excel数据
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:constructeScale:import")
    @RequestMapping(value = "import")
    public AjaxJson importBasic(@RequestParam("file") MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
        AjaxJson j = new AjaxJson();
        try {
            ImportUtil importUtil = new ImportUtil(file, 0, 0);
            //获取导入表格所有数据
            List<ConstructeScale> budgetStats = ExcelImportUtil.getConstructeUtil(importUtil);
            scaleService.insertList(budgetStats);
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入储备进度信息失败！失败信息：" + e.getMessage());
        }
        return j;
    }

    //导出excel
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:constructeScale:export")
    @RequestMapping(value = "export")
    public void exportExcel(ConstructeScale constructeScale, HttpServletRequest request, HttpServletResponse response) {
        List<ConstructeScale> scaleList = scaleService.exportFindList(constructeScale);

        //创建一个新的list来接收处理过后的数据
        List<ConstructeScale> list = new ArrayList<>();
        ConstructeScale scale = null;
        //赋值操作
        for (int i=0;i<scaleList.size();i++){
            scale = scaleList.get(i);
            //根据projectReserveId查出流程表的proc_ins_id
            String procInsId = scaleService.getByProjectReserveId(scale.getProjectReserveId());
            //根据proc_ins_id去运行时任务节点表（act_ru_task）中查出当前流程名称
            String name = scaleService.getByExecuteId(procInsId);
            //根据procInsId查出历史流程名称和时间
            List<History> histories = scaleService.getHistoryByExecuteId(procInsId);
            //当size>0时待办有历史流程，
            if (histories.size()>0){
                //设置项目进度
                scale.setReserveProgress(name);
                //根据相关规则设置物料信息
                //根物料规范的数据比对，判断在线开甲供材属于哪一类型的数据
                //先从物料详细数据中查出规范数据
                List<StandardDetail> detailList = scaleService.getStandardDetail();
                //在从在线开甲供材的表中查出所有数据
                List<MaterialData> materialDataList = scaleService.getMaterialData(scale.getProjectName());
                //进行对比
                if (materialDataList.size()>0){
                    double kgSum = 0;
                    double xlSum = 0;
                    double dlSum = 0;
                    double fzxSum = 0;
                    double hwxSum = 0;
                    double hwsSum = 0;
                    double zskgSum = 0;
                    double pdSum = 0;
                    double pdsSum = 0;
                    double xbSum = 0;
                    double xbkvaSum = 0;
                    double pdkvaSum = 0;
                    double pdkSum = 0;
                    double dyxlSum = 0;
                    double dydlSum = 0;
                    double dlfzxSum = 0;
                    double dlfzgSum = 0;
                    double pdftuSum = 0;
                    double pddtuSum = 0;
                    double glSum = 0;
                    double dltdSum = 0;
                    String num = "";
                    for (int j=0;j<materialDataList.size();j++){

                        for (int k=0;k<detailList.size();k++){

                            //通过物料编码和物料描述来进行对比
                            if (detailList.get(k).getMaterialCode().equals(materialDataList.get(j).getMaterialCode())
                                    && detailList.get(k).getMaterialDescription().equals(materialDataList.get(j).getMaterialName())){
                                if (detailList.get(k).getProjectCategory().equals("开关柜")){
                                    kgSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    scale.setSwitchCabinet(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("架空线路")){
                                    xlSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = xlSum + "";
                                    scale.setOverheadLine(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("电力电缆")){
                                    dlSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = dlSum + "";
                                    scale.setPowerCable(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("电缆分支箱")){
                                    fzxSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = fzxSum + "";
                                    scale.setCableBranchBox(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("户外环网箱")){
                                    hwxSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = hwxSum + "";
                                    scale.setOutdoorRingCage(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("环网箱/室")){
                                    hwsSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = hwsSum + "";
                                    scale.setRingRoom(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("柱上开关")){
                                    zskgSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = zskgSum + "";
                                    scale.setOnColumnSwitch(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("配电室kva")){
                                    pdSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = pdSum + "";
                                    scale.setSwitchRoomKVA(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("配电室")){
                                    pdsSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = pdsSum + "";
                                    scale.setSwitchRoom(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("箱变kva")){
                                    xbkvaSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = xbkvaSum + "";
                                    scale.setBoxChangeKVA(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("箱变")){
                                    xbSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = xbSum + "";
                                    scale.setBoxChange(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("配电变压器kva")){
                                    pdkvaSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = pdkvaSum + "";
                                    scale.setDistributionTransformerKVA(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("配电变压器")){
                                    pdkSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = pdkSum + "";
                                    scale.setDistributionTransformer(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("低压线路")){
                                    dyxlSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = dyxlSum + "";
                                    scale.setLowVoltageLine(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("低压电缆")){
                                    dydlSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = dydlSum + "";
                                    scale.setLowVoltageCable(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("低压电缆分支箱")){
                                    dlfzxSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = dlfzxSum + "";
                                    scale.setLowVoltageCableBox(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("低压配电箱（柜）")){
                                    dlfzgSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = dlfzgSum + "";
                                    scale.setLowVoltageCabinet(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("配电自动化终端（FTU）")){
                                    pdftuSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = pdftuSum + "";
                                    scale.setDistributeAutoMateTerminalFTU(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("配电自动化终端（DTU）")){
                                    pddtuSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = pddtuSum + "";
                                    scale.setDistributeAutoMateTerminalDTU(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("光缆")){
                                    glSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = glSum + "";
                                    scale.setOpticalCable(num);
                                }
                                if (detailList.get(k).getProjectCategory().equals("电缆通道")){
                                    dltdSum += Double.parseDouble(materialDataList.get(j).getTotalCount());
                                    num = dltdSum + "";
                                    scale.setCablePassage(num);
                                }
                            }
                        }
                    }
                }
            }else {
                //设置项目进度
                scale.setReserveProgress("");
            }
            list.add(scale);
        }
        //生成序列号
        int i = 1;
        for (ConstructeScale s : list) {
            s.setId(String.valueOf(i));
            i++;
        }

        ExcelExportUtil util = ExcelExportUtil.getOne();
        XSSFWorkbook workbook = util.getWorkbook("constructeScale");
        util.exportConstructe(response,workbook,list);
    }

}
