package com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.web;

import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.DemoliteList;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.service.DemoliteListService;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.util.ExcelExportUtil;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**
 * 储备项目拆旧物资汇总表
 * @author: 王强
 * @create: 2022-08-23 08:52
 **/
@Controller
@RequestMapping(value = "${adminPath}/cg/reservemanagementcenter/demoliteList")
public class DemoliteListController extends BaseController {

    @Autowired
    private DemoliteListService listService;

    @ModelAttribute
    public DemoliteList get(@RequestParam(required = false) String id) {
        DemoliteList entity = null;
        if (StringUtils.isNotBlank(id)) {
            entity = listService.get(id);
        }
        if (entity == null) {
            entity = new DemoliteList();
        }
        return entity;
    }

    /**
     * 储备项目拆旧物资汇总表页面
     * @param demoliteList
     * @param model
     * @return
     */
    @RequiresPermissions("cg:reservemanagementcenter:demoliteList:list")
    @RequestMapping(value = "demoliteList")
    public String demoliteList(DemoliteList demoliteList, Model model) {
        model.addAttribute("demoliteList", demoliteList);
        return "modules/cg/statisticalTable/demoliteList/demoliteList";
    }

    /**
     * 储备项目拆旧物资汇总列表
     * @param demoliteList
     * @param request
     * @param response
     * @param model
     * @return
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:demoliteList:list")
    @RequestMapping(value = "list")
    public Map<String, Object> basicData(DemoliteList demoliteList, HttpServletRequest request, HttpServletResponse response, Model model) {

        Page<DemoliteList> page = listService.findPage(new Page<DemoliteList>(request, response), demoliteList);
        return getBootstrapData(page);
    }

    /**
     * 删除表单
     */
    @ResponseBody
    @RequiresPermissions(value={"cg:reservemanagementcenter:demoliteList:del"},logical= Logical.OR)
    @RequestMapping(value = "deleteAll")
    public AjaxJson delete(String ids){
        AjaxJson j = new AjaxJson();
        String idArray[] =ids.split(",");
        for(String id : idArray){
            listService.delete(listService.get(id));
        }
        j.setMsg("删除储备项目拆旧物资汇总列表信息成功");
        return j;
    }

    //导出excel
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:demoliteList:export")
    @RequestMapping(value = "export")
    public void exportExcel(DemoliteList demoliteList, HttpServletRequest request, HttpServletResponse response) {
        List<DemoliteList> list = listService.exportFindList(demoliteList);

        //生成序列号
        int i = 1;
        for (DemoliteList s : list) {
            s.setId(String.valueOf(i));
            i++;
        }

        ExcelExportUtil util = ExcelExportUtil.getOne();
        XSSFWorkbook workbook = util.getWorkbook("demoliteList");
        util.exportDemoliteList(response,workbook,list);
    }
}
