package com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.web;

import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.EnginnerQuery;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.History;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.service.EnginnerQueryService;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.util.ExcelExportUtil;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 立杆和土建工程量统计
 * @author: 王强
 * @create: 2022-08-22 15:19
 **/
@Controller
@RequestMapping(value = "${adminPath}/cg/reservemanagementcenter/enginnerQuery")
public class EnginnerQueryController extends BaseController {

    @Autowired
    private EnginnerQueryService queryService;

    @ModelAttribute
    public EnginnerQuery get(@RequestParam(required = false) String id) {
        EnginnerQuery entity = null;
        if (StringUtils.isNotBlank(id)) {
            entity = queryService.get(id);
        }
        if (entity == null) {
            entity = new EnginnerQuery();
        }
        return entity;
    }

    /**
     * 立杆和土建工程量统计列表页面
     * @param enginnerQuery
     * @param model
     * @return
     */
    @RequiresPermissions("cg:reservemanagementcenter:enginnerQuery:list")
    @RequestMapping(value = "queryList")
    public String budgetList(EnginnerQuery enginnerQuery, Model model) {
        model.addAttribute("enginnerQuery", enginnerQuery);
        return "modules/cg/statisticalTable/enginnerQuery/enginnerQuery";
    }

    /**
     * 立杆和土建工程量统计数据列表
     * @param enginnerQuery
     * @param request
     * @param response
     * @param model
     * @return
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:enginnerQuery:list")
    @RequestMapping(value = "list")
    public Map<String, Object> basicData(EnginnerQuery enginnerQuery, HttpServletRequest request, HttpServletResponse response, Model model) {

        Page<EnginnerQuery> page = queryService.findPage(new Page<EnginnerQuery>(request, response), enginnerQuery);
        List<EnginnerQuery> list = page.getList();
        List<EnginnerQuery> queryList = new ArrayList<>();
        EnginnerQuery query = null;
        for (int i=0;i<list.size();i++){
            query = list.get(i);
            //根据projectReserveId查出流程表的proc_ins_id
            String procInsId = queryService.getByProjectReserveId(query.getProjectReserveId());
            //根据procInsId查出历史流程名称和时间
            List<History> histories = queryService.getHistoryByExecuteId(procInsId);
            //当size>0时待办有历史流程，
            if (histories.size()>0){
                //根据proc_ins_id去运行时任务节点表（act_ru_task）中查出当前流程名称
                String name = queryService.getByExecuteId(procInsId);
                query.setReserveProgress(name);
            }
            //设置立杆信息数据
            //根据物料编码查询物料总量
            Double sum1 = queryService.getData("500033660","","");
            Double sum2 = queryService.getData("500013972","","");
            Double sum3 = queryService.getData("500013974","","");
            Double sum4 = queryService.getData("500033743","","");
            Double sum5 = queryService.getData("乙供","等径水泥杆,300mm,9m,18φ16,下","");
            Double sum6 = queryService.getData("500124726","","");
            query.setPole190X10(sum1 == null ? "" : sum1 + "");
            query.setPole190X12(sum2 == null ? "" : sum2 + "");
            query.setPole190X15(sum3 == null ? "" : sum3 + "");
            query.setPole230X15("");
            query.setPole230X18(sum4 == null ? "" : sum4 + "");
            query.setPole300X9(sum5 == null ? "" : sum5 + "");
            query.setPole350X15(sum6 == null ? "" : sum6 + "");
            //设置钢管杆信息
            Double guan1 = queryService.getData("","配农网用,整套杆,GNH31-13-B","每基6.18吨（含基础+地脚螺栓）");
            Double guan2 = queryService.getData("","配农网用,整套杆,GNH31-13-B","每基2.78吨（不含基础，含地脚螺栓）");
            Double guan3 = queryService.getData("","配农网用,整套杆,GNH31-13-D","每基6.42吨（含基础+地脚螺栓）");
            Double guan4 = queryService.getData("","配农网用,整套杆,GNH31-13-D","每基3.02吨（不含基础，含地脚螺栓）");
            Double guan5 = queryService.getData("","配农网用,整套杆,GNH35-13-B","每基8.71吨（含基础+地脚螺栓）");
            Double guan6 = queryService.getData("","配农网用,整套杆,GNH35-13-B","每基4.01吨（不含基础，含地脚螺栓）");
            Double guan7 = queryService.getData("","配农网用,整套杆,GNH35-13-E","每基8.1吨（含基础+地脚螺栓）");
            Double guan8 = queryService.getData("","配农网用,整套杆,GNH35-13-E","每基4.31吨（不含基础，含地脚螺栓）");
            Double guan9 = queryService.getData("","配农网用,整套杆,GNH39-13-B","每基11.24吨（含基础+地脚螺栓）");
            Double guan10 = queryService.getData("","配农网用,整套杆,GNH39-13-B","每基5.8吨（不含基础，含地脚螺栓）");
            query.setSteelPipeGNH31Contains(guan1 == null ? "" : guan1 + "");
            query.setSteelPipeGNH31NoContains(guan2 == null ? "" : guan2 + "");
            query.setSteelPipeGNH3113DContains(guan3 == null ? "" : guan3 + "");
            query.setSteelPipeGNH3113DNoContains(guan4 == null ? "" : guan4 + "");
            query.setSteelPipeCNH35Contains(guan5 == null ? "" : guan5 + "");
            query.setSteelPipeCNH35NoContains(guan6 == null ? "" : guan6 + "");
            query.setSteelPipeCNH3513EContains(guan7 == null ? "" : guan7 + "");
            query.setSteelPipeCNH3513ENoContains(guan8 == null ? "" : guan8 + "");
            query.setSteelPipeCNH39Contains(guan9 == null ? "" : guan9 + "");
            query.setSteelPipeCNH39NoContains(guan10 == null ? "" : guan10 + "");
            query.setPrefabricatedWell("");
            query.setPrefabricatedBase("");
            query.setPrefabricatedPackage("");


            queryList.add(query);
        }
        page.setList(queryList);
        return getBootstrapData(page);
    }

    //导出excel
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:enginnerQuery:export")
    @RequestMapping(value = "export")
    public void exportExcel(EnginnerQuery enginnerQuery, HttpServletRequest request, HttpServletResponse response) {
        List<EnginnerQuery> list = queryService.exportFindList(enginnerQuery);

        List<EnginnerQuery> queryList = new ArrayList<>();
        EnginnerQuery query = null;
        for (int i=0;i<list.size();i++){
            query = list.get(i);
            //根据projectReserveId查出流程表的proc_ins_id
            String procInsId = queryService.getByProjectReserveId(query.getProjectReserveId());
            //根据procInsId查出历史流程名称和时间
            List<History> histories = queryService.getHistoryByExecuteId(procInsId);
            //当size>0时待办有历史流程，
            if (histories.size()>0){
                //根据proc_ins_id去运行时任务节点表（act_ru_task）中查出当前流程名称
                String name = queryService.getByExecuteId(procInsId);
                query.setReserveProgress(name);
            }
            //设置立杆信息数据
            //根据物料编码查询物料总量
            Double sum1 = queryService.getData("500033660","","");
            Double sum2 = queryService.getData("500013972","","");
            Double sum3 = queryService.getData("500013974","","");
            Double sum4 = queryService.getData("500033743","","");
            Double sum5 = queryService.getData("乙供","等径水泥杆,300mm,9m,18φ16,下","");
            Double sum6 = queryService.getData("500124726","","");
            query.setPole190X10(sum1 == null ? "" : sum1 + "");
            query.setPole190X12(sum2 == null ? "" : sum2 + "");
            query.setPole190X15(sum3 == null ? "" : sum3 + "");
            query.setPole230X15("");
            query.setPole230X18(sum4 == null ? "" : sum4 + "");
            query.setPole300X9(sum5 == null ? "" : sum5 + "");
            query.setPole350X15(sum6 == null ? "" : sum6 + "");
            //设置钢管杆信息
            Double guan1 = queryService.getData("","配农网用,整套杆,GNH31-13-B","每基6.18吨（含基础+地脚螺栓）");
            Double guan2 = queryService.getData("","配农网用,整套杆,GNH31-13-B","每基2.78吨（不含基础，含地脚螺栓）");
            Double guan3 = queryService.getData("","配农网用,整套杆,GNH31-13-D","每基6.42吨（含基础+地脚螺栓）");
            Double guan4 = queryService.getData("","配农网用,整套杆,GNH31-13-D","每基3.02吨（不含基础，含地脚螺栓）");
            Double guan5 = queryService.getData("","配农网用,整套杆,GNH35-13-B","每基8.71吨（含基础+地脚螺栓）");
            Double guan6 = queryService.getData("","配农网用,整套杆,GNH35-13-B","每基4.01吨（不含基础，含地脚螺栓）");
            Double guan7 = queryService.getData("","配农网用,整套杆,GNH35-13-E","每基8.1吨（含基础+地脚螺栓）");
            Double guan8 = queryService.getData("","配农网用,整套杆,GNH35-13-E","每基4.31吨（不含基础，含地脚螺栓）");
            Double guan9 = queryService.getData("","配农网用,整套杆,GNH39-13-B","每基11.24吨（含基础+地脚螺栓）");
            Double guan10 = queryService.getData("","配农网用,整套杆,GNH39-13-B","每基5.8吨（不含基础，含地脚螺栓）");
            query.setSteelPipeGNH31Contains(guan1 == null ? "" : guan1 + "");
            query.setSteelPipeGNH31NoContains(guan2 == null ? "" : guan2 + "");
            query.setSteelPipeGNH3113DContains(guan3 == null ? "" : guan3 + "");
            query.setSteelPipeGNH3113DNoContains(guan4 == null ? "" : guan4 + "");
            query.setSteelPipeCNH35Contains(guan5 == null ? "" : guan5 + "");
            query.setSteelPipeCNH35NoContains(guan6 == null ? "" : guan6 + "");
            query.setSteelPipeCNH3513EContains(guan7 == null ? "" : guan7 + "");
            query.setSteelPipeCNH3513ENoContains(guan8 == null ? "" : guan8 + "");
            query.setSteelPipeCNH39Contains(guan9 == null ? "" : guan9 + "");
            query.setSteelPipeCNH39NoContains(guan10 == null ? "" : guan10 + "");
            query.setPrefabricatedWell("");
            query.setPrefabricatedBase("");
            query.setPrefabricatedPackage("");


            queryList.add(query);
        }

        //生成序列号
        int i = 1;
        for (EnginnerQuery s : queryList) {
            s.setId(String.valueOf(i));
            i++;
        }

        ExcelExportUtil util = ExcelExportUtil.getOne();
        XSSFWorkbook workbook = util.getWorkbook("enginnerQuery");
        util.exportEnginnerQuery(response,workbook,queryList);
    }
}
