package com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.web;

import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.entity.ReserveProcessProject;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.entity.ReserveSummary;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.service.ReserveProcessProjectService;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.service.SummaryStatisticService;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.*;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.service.ProgressStatsService;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.util.ExcelExportUtil;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.util.ExcelImportUtil;
import com.jeeplus.modules.sg.managementcenter.materialproject.util.ImportUtil;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author: 王强
 * @create: 2022-08-18 09:20
 **/
@Controller
@RequestMapping(value = "${adminPath}/cg/reservemanagementcenter/progressStats")
public class ProgressStatsController extends BaseController {

    @Autowired
    private ProgressStatsService statsService;

    @Autowired
    private SummaryStatisticService statisticService;

    @Autowired
    private ReserveProcessProjectService projectService;

    @ModelAttribute
    public ProgressStats get(@RequestParam(required = false) String id) {
        ProgressStats entity = null;
        if (StringUtils.isNotBlank(id)) {
            entity = statsService.get(id);
        }
        if (entity == null) {
            entity = new ProgressStats();
        }
        return entity;
    }

    /**
     * 储备项目概预算列表页面
     * @param progressStats
     * @param model
     * @return
     */
    @RequiresPermissions("cg:reservemanagementcenter:progressStats:list")
    @RequestMapping(value = "progressList")
    public String progressList(ProgressStats progressStats, Model model) {
        model.addAttribute("progressStats", progressStats);
        return "modules/cg/statisticalTable/progressStats/progressStats";
    }

    /**
     * 储备进度统计表数据列表
     * @param progressStats
     * @param request
     * @param response
     * @param model
     * @return
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:progressStats:list")
    @RequestMapping(value = "list")
    public Map<String, Object> basicData(ProgressStats progressStats, HttpServletRequest request, HttpServletResponse response, Model model) {

        Page<ProgressStats> page = statsService.findPage(new Page<ProgressStats>(request, response), progressStats);
        List<ProgressStats> list = page.getList();
        List<ProgressStats> statsList = new ArrayList<>();
        ProgressStats stats = null;
        for (int i=0;i<list.size();i++){
            //进行赋值操作
            //查出proc_ins_id
            stats = list.get(i);
            String procInsId = statsService.getByProjectReserveId(stats.getProjectReserveId());
            //设置是否纳入实施计划 及 初定实施批次 及是否纳入储备
            ReserveSummary summary = new ReserveSummary();
            summary.setProjectReserveId(stats.getProjectReserveId());
            ReserveSummary reserveSummary = statisticService.queryByProjectReserveId(summary);
            stats.setImplementPlan(reserveSummary.getImplementPlan());
            stats.setInitialImplementBatch(reserveSummary.getInitialImplementBatch());
            stats.setIncludeReserve(reserveSummary.getIncludeReserve());
            //设置项目名称，储备项目类型，需求提出单位，储备批次，备注
            ReserveProcessProject process = new ReserveProcessProject();
            process.setProjectReserveId(stats.getProjectReserveId());
            ReserveProcessProject project = projectService.queryByProjectReserveId(process);
            stats.setProjectName(project.getProjectName());
            stats.setRequirementType(project.getRequirementType());
            stats.setDemandUnit(project.getDemandUnit());
            stats.setReserveBatches(project.getReserveBatches());
            stats.setBasicRemarks(project.getBasicRemarks());
            //设置设计委托发出时间 及 是否发出设计委托
            //查看历史流程节点信息
            //根据procInsId查出历史流程名称和时间
            List<History> histories = statsService.getHistoryByExecuteId(procInsId);
            //没有流程的情况
            if (histories.size()==0){
                stats.setReserveProgress("");
                //设置是否发出设计委托
                stats.setDesignCommission("否");
                stats.setDesignCommissionIssueTime(null);
                //设置初设提交时间 及 是否完成初设
                stats.setCompleteDesign("否");
                stats.setSubmitTime(null);
                //设置是否完成审图，是否通过图纸审核，未通过原因分析，通过审核日期，是否需设计变更
                stats.setCompleteDraw("");
                stats.setDrawAudit("");
                stats.setReasonForFailure("");
                stats.setPassAuditDate(null);
                stats.setChangeDesign("");
                //设置是否完成审核，是否通过专家审核，未通过原因分析，通过审核日期，是否需设计变更
                stats.setCompleteReview("");
                stats.setExpertReview("");
                stats.setEstimateReasonForFailure("");
                stats.setEstimatePassAuditDate(null);
                stats.setEstimateChangeDesign("");
                //设置图纸是否定稿，定稿日期
                stats.setDrawingFinalize("");
                stats.setFinalDate(null);
            }else {
                //根据procInsId查出当前流程名称
                String name = statsService.getByExecuteId(procInsId);
                if (name == null){
                    stats.setReserveProgress("流程结束");
                }else {
                    //设置储备进度
                    stats.setReserveProgress(name);
                }
                //有流程的情况
                String startTime = histories.get(histories.size() - 1).getStartTime();
                stats.setDesignCommission("是");
                stats.setDesignCommissionIssueTime(startTime);
                //设置初设提交时间 及 是否完成初设
                //根据节点名称查找
                List<History> history = statsService.getHistoryByActName(procInsId, "设计单位上传施工图纸");
                String endTime = history.get(history.size() - 1).getEndTime();
                if (endTime != null){
                    stats.setCompleteDesign("是");
                    stats.setSubmitTime(endTime);
                }else {
                    stats.setCompleteDesign("否");
                    stats.setSubmitTime(null);
                }
                //设置是否完成审图，是否通过图纸审核，未通过原因分析，通过审核日期，是否需设计变更
                //在历史表中根据节点名称查找
                List<History> history02 = statsService.getHistoryByActName(procInsId, "联合审图配网项目经理审图");
                if (history02.size()>0){
                    String endTime02 = history02.get(0).getEndTime();
                    //刚进入联合审图的情况
                    if (endTime02 == null){
                        stats.setCompleteDraw("");
                        stats.setDrawAudit("");
                        stats.setReasonForFailure("");
                        stats.setPassAuditDate(null);
                        stats.setChangeDesign("");
                    }else {
                        //根据时间来查找下一个节点的(联合审图下一步)信息
                        History historyByTime = statsService.getHistoryByTime(endTime02, procInsId);
                        HistoryComment comment = statsService.getCommentByTime(endTime02, procInsId);
                        if (historyByTime != null){
                            //通过的情况
                            if (historyByTime.getName().equals("设计单位上传预算书")){
                                stats.setCompleteDraw("是");
                                stats.setDrawAudit("是");
                                stats.setReasonForFailure("");
                                stats.setPassAuditDate(endTime02);
                                stats.setChangeDesign("否");
                            }else if (historyByTime.getName().equals("设计单位上传施工图纸")){
                                //不通过
                                stats.setCompleteDraw("否");
                                stats.setDrawAudit("不通过");
                                stats.setReasonForFailure(comment.getMessage());
                                stats.setPassAuditDate(endTime02);
                                stats.setChangeDesign("是");
                            }
                        }else {
                            //还有一种是  取消、待定的情况  流程结束
                            stats.setCompleteDraw("否");
                            //判断是哪条流程线
                            HistoryVarinst byProcInstId = statsService.getByProcInstId(endTime02, procInsId);
                            if (byProcInstId.getName().equals("no")){
                                stats.setDrawAudit("取消");
                                stats.setReasonForFailure(comment.getMessage());
                                stats.setPassAuditDate(endTime02);
                                stats.setChangeDesign("是");
                            }else if (byProcInstId.getName().equals("rural")){
                                stats.setDrawAudit("待定");
                                stats.setReasonForFailure(comment.getMessage());
                                stats.setPassAuditDate(endTime02);
                                stats.setChangeDesign("是");
                            }
                        }
                    }
                }else {
                    stats.setCompleteDraw("");
                    stats.setDrawAudit("");
                    stats.setReasonForFailure("");
                    stats.setPassAuditDate(null);
                    stats.setChangeDesign("");
                }
                //设置是否完成审核，是否通过专家审核，未通过原因分析，通过审核日期，是否需设计变更  概预算审核
                //在历史表中根据节点名称查找是否就行了概预算审核
                List<History> history03 = statsService.getHistoryByActName(procInsId, "技经人员审预算");
                if (history03.size()>0){
                    String endTime03 = history03.get(0).getEndTime();
                    //刚进入审预算的情况
                    if (endTime03 == null){
                        stats.setCompleteReview("");
                        stats.setExpertReview("");
                        stats.setEstimateReasonForFailure("");
                        stats.setEstimatePassAuditDate(null);
                        stats.setEstimateChangeDesign("");
                        //设置图纸是否定稿，定稿日期
                        stats.setDrawingFinalize("");
                        stats.setFinalDate(null);
                    }else {
                        //根据时间来查找下一个节点的信息（是通过还是驳回）
                        History hi = statsService.getHistoryByTime(endTime03,procInsId);
                        HistoryComment comment = statsService.getCommentByTime(endTime03, procInsId);
                        //通过的情况
                        if (hi == null){
                            stats.setCompleteReview("是");
                            stats.setExpertReview("是");
                            stats.setEstimateReasonForFailure("");
                            stats.setEstimatePassAuditDate(endTime03);
                            stats.setEstimateChangeDesign("否");
                            //设置图纸是否定稿，定稿日期
                            stats.setDrawingFinalize("是");
                            stats.setFinalDate(endTime03);
                        }else {
                            //驳回的情况
                            stats.setCompleteReview("否");
                            stats.setExpertReview("否");
                            stats.setEstimateReasonForFailure(comment.getMessage());
                            stats.setEstimatePassAuditDate(endTime03);
                            stats.setEstimateChangeDesign("是");
                            //设置图纸是否定稿，定稿日期
                            stats.setDrawingFinalize("否");
                            stats.setFinalDate("");
                        }
                    }
                }else {
                    stats.setCompleteReview("");
                    stats.setExpertReview("");
                    stats.setEstimateReasonForFailure("");
                    stats.setEstimatePassAuditDate(null);
                    stats.setEstimateChangeDesign("");
                    //设置图纸是否定稿，定稿日期
                    stats.setDrawingFinalize("");
                    stats.setFinalDate(null);
                }
            }

            statsList.add(stats);
        }
        page.setList(statsList);
        System.out.println(list);
        return getBootstrapData(page);
    }

    /**
     * 删除表单
     */
    @ResponseBody
    @RequiresPermissions(value={"cg:reservemanagementcenter:progressStats:del"},logical= Logical.OR)
    @RequestMapping(value = "deleteAll")
    public AjaxJson delete(String ids){
        AjaxJson j = new AjaxJson();
        String idArray[] =ids.split(",");
        for(String id : idArray){
            statsService.delete(statsService.get(id));
        }
        j.setMsg("删除储备进度信息成功");
        return j;
    }

    /**
     *  导入Excel数据
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:progressStats:import")
    @RequestMapping(value = "import")
    public AjaxJson importBasic(@RequestParam("file") MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
        AjaxJson j = new AjaxJson();
        try {
            ImportUtil importUtil = new ImportUtil(file, 0, 0);
            //获取导入表格所有数据
            List<ProgressStats> budgetStats = ExcelImportUtil.getProgressUtil(importUtil);
            statsService.insertList(budgetStats);
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入储备进度信息失败！失败信息：" + e.getMessage());
        }
        return j;
    }

    //导出excel
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:progressStats:export")
    @RequestMapping(value = "export")
    public void exportExcel(ProgressStats progressStats, HttpServletRequest request, HttpServletResponse response) {
        List<ProgressStats> list = statsService.exportFindList(progressStats);

        List<ProgressStats> statsList = new ArrayList<>();
        ProgressStats stats = null;
        for (int i=0;i<list.size();i++){
            //进行赋值操作
            //查出proc_ins_id
            stats = list.get(i);
            String procInsId = statsService.getByProjectReserveId(stats.getProjectReserveId());
            //设置是否纳入实施计划 及 初定实施批次 及是否纳入储备
            ReserveSummary summary = new ReserveSummary();
            summary.setProjectReserveId(stats.getProjectReserveId());
            ReserveSummary reserveSummary = statisticService.queryByProjectReserveId(summary);
            stats.setImplementPlan(reserveSummary.getImplementPlan());
            stats.setInitialImplementBatch(reserveSummary.getInitialImplementBatch());
            stats.setIncludeReserve(reserveSummary.getIncludeReserve());
            //设置项目名称，储备项目类型，需求提出单位，储备批次，备注
            ReserveProcessProject process = new ReserveProcessProject();
            process.setProjectReserveId(stats.getProjectReserveId());
            ReserveProcessProject project = projectService.queryByProjectReserveId(process);
            stats.setProjectName(project.getProjectName());
            stats.setRequirementType(project.getRequirementType());
            stats.setDemandUnit(project.getDemandUnit());
            stats.setReserveBatches(project.getReserveBatches());
            stats.setBasicRemarks(project.getBasicRemarks());
            //设置设计委托发出时间 及 是否发出设计委托
            //查看历史流程节点信息
            //根据procInsId查出历史流程名称和时间
            List<History> histories = statsService.getHistoryByExecuteId(procInsId);
            //没有流程的情况
            if (histories.size()==0){
                //设置是否发出设计委托
                stats.setDesignCommission("否");
                stats.setDesignCommissionIssueTime(null);
                //设置初设提交时间 及 是否完成初设
                stats.setCompleteDesign("否");
                stats.setSubmitTime(null);
                //设置是否完成审图，是否通过图纸审核，未通过原因分析，通过审核日期，是否需设计变更
                stats.setCompleteDraw("");
                stats.setDrawAudit("");
                stats.setReasonForFailure("");
                stats.setPassAuditDate(null);
                stats.setChangeDesign("");
                //设置是否完成审核，是否通过专家审核，未通过原因分析，通过审核日期，是否需设计变更
                stats.setCompleteReview("");
                stats.setExpertReview("");
                stats.setEstimateReasonForFailure("");
                stats.setEstimatePassAuditDate(null);
                stats.setEstimateChangeDesign("");
                //设置图纸是否定稿，定稿日期
                stats.setDrawingFinalize("");
                stats.setFinalDate(null);
            }else {
                //根据procInsId查出当前流程名称
                String name = statsService.getByExecuteId(procInsId);
                if (name == null){
                    stats.setReserveProgress("流程结束");
                }else {
                    //设置储备进度
                    stats.setReserveProgress(name);
                }
                //有流程的情况
                String startTime = histories.get(histories.size() - 1).getStartTime();
                stats.setDesignCommission("是");
                stats.setDesignCommissionIssueTime(startTime);
                //设置初设提交时间 及 是否完成初设
                //根据节点名称查找
                List<History> history = statsService.getHistoryByActName(procInsId, "设计单位上传施工图纸");
                String endTime = history.get(history.size() - 1).getEndTime();
                if (endTime != null){
                    stats.setCompleteDesign("是");
                    stats.setSubmitTime(endTime);
                }else {
                    stats.setCompleteDesign("否");
                    stats.setSubmitTime(null);
                }
                //设置是否完成审图，是否通过图纸审核，未通过原因分析，通过审核日期，是否需设计变更
                //在历史表中根据节点名称查找
                List<History> history02 = statsService.getHistoryByActName(procInsId, "联合审图配网项目经理审图");
                if (history02.size()>0){
                    String endTime02 = history02.get(0).getEndTime();
                    //刚进入联合审图的情况
                    if (endTime02 == null){
                        stats.setCompleteDraw("");
                        stats.setDrawAudit("");
                        stats.setReasonForFailure("");
                        stats.setPassAuditDate(null);
                        stats.setChangeDesign("");
                    }else {
                        //根据时间来查找下一个节点的(联合审图下一步)信息
                        History historyByTime = statsService.getHistoryByTime(endTime02, procInsId);
                        HistoryComment comment = statsService.getCommentByTime(endTime02, procInsId);
                        if (historyByTime != null){
                            //通过的情况
                            if (historyByTime.getName().equals("设计单位上传预算书")){
                                stats.setCompleteDraw("是");
                                stats.setDrawAudit("是");
                                stats.setReasonForFailure("");
                                stats.setPassAuditDate(endTime02);
                                stats.setChangeDesign("否");
                            }else if (historyByTime.getName().equals("设计单位上传施工图纸")){
                                //不通过
                                stats.setCompleteDraw("否");
                                stats.setDrawAudit("不通过");
                                stats.setReasonForFailure(comment.getMessage());
                                stats.setPassAuditDate(endTime02);
                                stats.setChangeDesign("是");
                            }
                        }else {
                            //还有一种是  取消、待定的情况  流程结束
                            stats.setCompleteDraw("否");
                            //判断是哪条流程线
                            HistoryVarinst byProcInstId = statsService.getByProcInstId(endTime02, procInsId);
                            if (byProcInstId.getName().equals("no")){
                                stats.setDrawAudit("取消");
                                stats.setReasonForFailure(comment.getMessage());
                                stats.setPassAuditDate(endTime02);
                                stats.setChangeDesign("是");
                            }else if (byProcInstId.getName().equals("rural")){
                                stats.setDrawAudit("待定");
                                stats.setReasonForFailure(comment.getMessage());
                                stats.setPassAuditDate(endTime02);
                                stats.setChangeDesign("是");
                            }
                        }
                    }
                }else {
                    stats.setCompleteDraw("");
                    stats.setDrawAudit("");
                    stats.setReasonForFailure("");
                    stats.setPassAuditDate(null);
                    stats.setChangeDesign("");
                }
                //设置是否完成审核，是否通过专家审核，未通过原因分析，通过审核日期，是否需设计变更  概预算审核
                //在历史表中根据节点名称查找是否就行了概预算审核
                List<History> history03 = statsService.getHistoryByActName(procInsId, "技经人员审预算");
                if (history03.size()>0){
                    String endTime03 = history03.get(0).getEndTime();
                    //刚进入审预算的情况
                    if (endTime03 == null){
                        stats.setCompleteReview("");
                        stats.setExpertReview("");
                        stats.setEstimateReasonForFailure("");
                        stats.setEstimatePassAuditDate(null);
                        stats.setEstimateChangeDesign("");
                        //设置图纸是否定稿，定稿日期
                        stats.setDrawingFinalize("");
                        stats.setFinalDate(null);
                    }else {
                        //根据时间来查找下一个节点的信息（是通过还是驳回）
                        History hi = statsService.getHistoryByTime(endTime03,procInsId);
                        HistoryComment comment = statsService.getCommentByTime(endTime03, procInsId);
                        //通过的情况
                        if (hi == null){
                            stats.setCompleteReview("是");
                            stats.setExpertReview("是");
                            stats.setEstimateReasonForFailure("");
                            stats.setEstimatePassAuditDate(endTime03);
                            stats.setEstimateChangeDesign("否");
                            //设置图纸是否定稿，定稿日期
                            stats.setDrawingFinalize("是");
                            stats.setFinalDate(endTime03);
                        }else {
                            //驳回的情况
                            stats.setCompleteReview("否");
                            stats.setExpertReview("否");
                            stats.setEstimateReasonForFailure(comment.getMessage());
                            stats.setEstimatePassAuditDate(endTime03);
                            stats.setEstimateChangeDesign("是");
                            //设置图纸是否定稿，定稿日期
                            stats.setDrawingFinalize("否");
                            stats.setFinalDate("");
                        }
                    }
                }else {
                    stats.setCompleteReview("");
                    stats.setExpertReview("");
                    stats.setEstimateReasonForFailure("");
                    stats.setEstimatePassAuditDate(null);
                    stats.setEstimateChangeDesign("");
                    //设置图纸是否定稿，定稿日期
                    stats.setDrawingFinalize("");
                    stats.setFinalDate(null);
                }
            }

            statsList.add(stats);
        }
        //生成序列号
        int i = 1;
        for (ProgressStats s : statsList) {
            s.setId(String.valueOf(i));
            i++;
        }

        ExcelExportUtil util = ExcelExportUtil.getOne();
        XSSFWorkbook workbook = util.getWorkbook("progressStats");
        util.exportProgress(response,workbook,statsList);
    }
}
