package com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.web;

import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.History;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.SpecialSupply;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.service.SpecialSupplyService;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.util.ExcelExportUtil;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 特殊物资统计
 * @author: 王强
 * @create: 2022-08-22 17:58
 **/
@Controller
@RequestMapping(value = "${adminPath}/cg/reservemanagementcenter/specialSupply")
public class SpecialSupplyController extends BaseController {

    @Autowired
    private SpecialSupplyService supplyService;

    @ModelAttribute
    public SpecialSupply get(@RequestParam(required = false) String id) {
        SpecialSupply entity = null;
        if (StringUtils.isNotBlank(id)) {
            entity = supplyService.get(id);
        }
        if (entity == null) {
            entity = new SpecialSupply();
        }
        return entity;
    }

    /**
     * 特殊物资统计表页面
     * @param specialSupply
     * @param model
     * @return
     */
    @RequiresPermissions("cg:reservemanagementcenter:specialSupply:list")
    @RequestMapping(value = "supplyList")
    public String supplyList(SpecialSupply specialSupply, Model model) {
        model.addAttribute("specialSupply", specialSupply);
        return "modules/cg/statisticalTable/specialSupply/specialSupply";
    }

    /**
     * 特殊物资统计数据列表
     * @param specialSupply
     * @param request
     * @param response
     * @param model
     * @return
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:specialSupply:list")
    @RequestMapping(value = "list")
    public Map<String, Object> basicData(SpecialSupply specialSupply, HttpServletRequest request, HttpServletResponse response, Model model) {

        Page<SpecialSupply> page = supplyService.findPage(new Page<SpecialSupply>(request, response), specialSupply);
        List<SpecialSupply> list = page.getList();
        List<SpecialSupply> supplyList = new ArrayList<>();
        SpecialSupply supply = null;
        for (int i=0;i<list.size();i++){
            supply = list.get(i);
            //进行赋值操作
            //根据projectReserveId查出流程表的proc_ins_id
            String procInsId = supplyService.getByProjectReserveId(supply.getProjectReserveId());
            //根据procInsId查出历史流程名称和时间
            List<History> histories = supplyService.getHistoryByExecuteId(procInsId);
            //当size>0时待办有历史流程，
            if (histories.size()>0){
                //根据proc_ins_id去运行时任务节点表（act_ru_task）中查出当前流程名称
                String name = supplyService.getByExecuteId(procInsId);
                supply.setReserveProgress(name);
            }
            //设置20kv物资统计
            //设置高低压开关柜统计
            Double kgg1 = supplyService.getData("500116761", "环网柜,AC10kV,630A,负荷开关柜,SF6,户内。SF6灭弧,带避雷器,电动,端子排", "");
            Double kgg2 = supplyService.getData("500116761", "环网柜,AC10kV,630A,负荷开关柜,SF6,户内。SF6灭弧,不带避雷器,电动,端子排", "");
            Double kgg3 = supplyService.getData("500006630", "", "");
            Double kgg4 = supplyService.getData("500006556", "", "");
            Double kgg5 = supplyService.getData("500006703", "", "");
            Double kgg6 = supplyService.getData("500107555", "", "");
            Double kgg7 = supplyService.getData("500107558", "", "");
            Double kgg8 = supplyService.getData("500107562", "", "");
            supply.setSwitchCabinet10KV(kgg1 == null ? "" : kgg1 + "");
            supply.setSwitchCabinet10KV02(kgg2 == null ? "" : kgg2 + "");
            supply.setSwitchCabinetIncomeLine(kgg3 == null ? "" : kgg3 + "");
            supply.setSwitchCabinetSegment(kgg4 == null ? "" : kgg4 + "");
            supply.setSwitchCabinetFeeder(kgg5 == null ? "" : kgg5 + "");
            supply.setNumberOfBranchBoxFourWay(kgg6 == null ? "" : kgg6 + "");
            supply.setNumberOfBranchBoxFiveWay(kgg7 == null ? "" : kgg7 + "");
            supply.setNumberOfBranchBoxSevenWay(kgg8 == null ? "" : kgg8 + "");
            //设置低压分支箱路上统计
            //设置箱变路数统计
            //设置特殊尺寸物资统计

            supplyList.add(supply);
        }
        page.setList(supplyList);
        return getBootstrapData(page);
    }

    //导出excel
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:specialSupply:export")
    @RequestMapping(value = "export")
    public void exportExcel(SpecialSupply specialSupply, HttpServletRequest request, HttpServletResponse response) {
        List<SpecialSupply> list = supplyService.exportFindList(specialSupply);

        List<SpecialSupply> supplyList = new ArrayList<>();
        SpecialSupply supply = null;
        for (int i=0;i<list.size();i++){
            supply = list.get(i);
            //进行赋值操作
            //根据projectReserveId查出流程表的proc_ins_id
            String procInsId = supplyService.getByProjectReserveId(supply.getProjectReserveId());
            //根据procInsId查出历史流程名称和时间
            List<History> histories = supplyService.getHistoryByExecuteId(procInsId);
            //当size>0时待办有历史流程，
            if (histories.size()>0){
                //根据proc_ins_id去运行时任务节点表（act_ru_task）中查出当前流程名称
                String name = supplyService.getByExecuteId(procInsId);
                supply.setReserveProgress(name);
            }
            //设置20kv物资统计
            //设置高低压开关柜统计
            Double kgg1 = supplyService.getData("500116761", "环网柜,AC10kV,630A,负荷开关柜,SF6,户内。SF6灭弧,带避雷器,电动,端子排", "");
            Double kgg2 = supplyService.getData("500116761", "环网柜,AC10kV,630A,负荷开关柜,SF6,户内。SF6灭弧,不带避雷器,电动,端子排", "");
            Double kgg3 = supplyService.getData("500006630", "", "");
            Double kgg4 = supplyService.getData("500006556", "", "");
            Double kgg5 = supplyService.getData("500006703", "", "");
            Double kgg6 = supplyService.getData("500107555", "", "");
            Double kgg7 = supplyService.getData("500107558", "", "");
            Double kgg8 = supplyService.getData("500107562", "", "");
            supply.setSwitchCabinet10KV(kgg1 == null ? "" : kgg1 + "");
            supply.setSwitchCabinet10KV02(kgg2 == null ? "" : kgg2 + "");
            supply.setSwitchCabinetIncomeLine(kgg3 == null ? "" : kgg3 + "");
            supply.setSwitchCabinetSegment(kgg4 == null ? "" : kgg4 + "");
            supply.setSwitchCabinetFeeder(kgg5 == null ? "" : kgg5 + "");
            supply.setNumberOfBranchBoxFourWay(kgg6 == null ? "" : kgg6 + "");
            supply.setNumberOfBranchBoxFiveWay(kgg7 == null ? "" : kgg7 + "");
            supply.setNumberOfBranchBoxSevenWay(kgg8 == null ? "" : kgg8 + "");
            //设置低压分支箱路上统计
            //设置箱变路数统计
            //设置特殊尺寸物资统计

            supplyList.add(supply);
        }

        //生成序列号
        int i = 1;
        for (SpecialSupply s : supplyList) {
            s.setId(String.valueOf(i));
            i++;
        }

        ExcelExportUtil util = ExcelExportUtil.getOne();
        XSSFWorkbook workbook = util.getWorkbook("specialSupply");
        util.exportSpecialSupply(response,workbook,supplyList);
    }
}
